/* $Id: AddrBookCtrl.cpp,v 1.19 2007/02/01 19:46:26 olle Exp $
 *
 * Copyright (c) 2006, olle@toolcrypt.org
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 *    * Redistributions of source code must retain the above copyright notice,
 *      this list of conditions and the following disclaimer.
 *    * Redistributions in binary form must reproduce the above copyright notice,
 *      this list of conditions and the following disclaimer in the documentation
 *      and/or other materials provided with the distribution.
 *    * Neither the name of the <ORGANIZATION> nor the names of its contributors may
 *      be used to endorse or promote products derived from this software without 
 *      specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include "stdafx.h"
#include "AddrBookCtrl.h"
#include "ComboInListView.h"

#include "resource.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


CAddrBookCtrl::CAddrBookCtrl()
{
	m_bDrag = FALSE;
}

CAddrBookCtrl::~CAddrBookCtrl()
{
}

BEGIN_MESSAGE_MAP(CAddrBookCtrl, CSuperGridCtrl)
	//{{AFX_MSG_MAP(CAddrBookCtrl)
	ON_WM_CREATE()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CAddrBookCtrl message handlers

int CAddrBookCtrl::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CSuperGridCtrl::OnCreate(lpCreateStruct) == -1)
		return -1;
	/////////////////////
	//remember this	
	///////////////////
	//associate imagelist with listviewctrl	
	if(!m_image.Create(IDB_FOLDERS,16,1,RGB(0, 255, 255)))
		return -1;

	SetImageList(&m_image, LVSIL_SMALL);
	CImageList *pImageList = GetImageList(LVSIL_SMALL);
	if(pImageList)
		ImageList_GetIconSize(pImageList->m_hImageList, &m_cxImage, &m_cyImage);
	else
		return -1;
	
	return 0;
}

BOOL CAddrBookCtrl::SaveData(LPSTR filename)
{
	// delete old savefile
	DeleteFile(filename);

	// loop all root nodes
	POSITION pos = GetRootHeadPosition();
	while(pos != NULL)
	{
		CTreeItem *pSystemNode = (CTreeItem*)GetNextRoot(pos);
		if (!pSystemNode)
		{
			MessageBox("Error getting system node in CAddrBookCtrl::SaveData()!");
			return FALSE;
		}
		CItemInfo *pSystem = GetData(pSystemNode);
		if (!pSystem)
		{
			MessageBox("Error getting system node data in CAddrBookCtrl::SaveData()!");
			return FALSE;
		}

		// create the system
		CString sysid = pSystem->GetSubItem(0);
		WritePrivateProfileString(sysid, "sysName", pSystem->GetItemText(), filename);
		WritePrivateProfileString(sysid, "sysComment", pSystem->GetSubItem(1), filename);

		// process fleets for system
		const UINT nFleets = NumChildren(pSystemNode);
		if (nFleets > 0)
		{
			// write nrOfFleets for the system
			CString s;
			s.Format("%u", nFleets);
			WritePrivateProfileString(sysid, "numberOfFleets", s, filename);

			// loop all fleets in system
			POSITION pos = GetHeadPosition(pSystemNode);
			UINT fleetNr = 0;
			while (pos)
			{
				CTreeItem *pFleetNode = GetNextChild(pSystemNode, pos);
				if (!pFleetNode)
				{
					MessageBox("Error getting fleet node in CAddrBookCtrl::SaveData()!");
					return FALSE;
				}
				CItemInfo* pFleet = GetData(pFleetNode);
				if (!pFleet)
				{
					MessageBox("Error getting fleet node data in CAddrBookCtrl::SaveData()!");
					return FALSE;
				}

				// make profile labels for this fleet
				CString fleetIdLabel;
				fleetIdLabel.Format("fleet%u-Id", fleetNr);
				CString fleetNameLabel;
				fleetNameLabel.Format("fleet%u-Name", fleetNr);
				CString fleetCommentLabel;
				fleetCommentLabel.Format("fleet%u-Comment", fleetNr);

				// create the fleet keys
				WritePrivateProfileString(sysid, fleetNameLabel, pFleet->GetItemText(), filename);
				WritePrivateProfileString(sysid, fleetIdLabel, pFleet->GetSubItem(0), filename);
				WritePrivateProfileString(sysid, fleetCommentLabel, pFleet->GetSubItem(1), filename);

				// process addresses for this fleet
				const UINT nAddrs = NumChildren(pFleetNode);
				if (nAddrs > 0)
				{
					// write nrOfFleets for the fleet
					CString nrOfAddrsLabel;
					nrOfAddrsLabel.Format("fleet%u-NrOfUnits", fleetNr);
					CString s;
					s.Format("%u", nAddrs);
					WritePrivateProfileString(sysid, nrOfAddrsLabel, s, filename);

					// loop addresses in this fleet
					POSITION pos = GetHeadPosition(pFleetNode);
					UINT addrNr = 0;
					while (pos)
					{
						CTreeItem *pAddrNode = GetNextChild(pFleetNode, pos);
						if (!pAddrNode)
						{
							MessageBox("Error getting addr node in CAddrBookCtrl::SaveData()!");
							return FALSE;
						}
						CItemInfo* pAddr = GetData(pAddrNode);
						if (!pAddr)
						{
							MessageBox("Error getting addr node data in CAddrBookCtrl::SaveData()!");
							return FALSE;
						}

						// make profile labels for this unit address
						CString addrIdLabel;
						addrIdLabel.Format("fleet%u-unit%u-Id", fleetNr, addrNr);
						CString addrNameLabel;
						addrNameLabel.Format("fleet%u-unit%u-Name", fleetNr, addrNr);
						CString addrCommentLabel;
						addrCommentLabel.Format("fleet%u-unit%u-Comment", fleetNr, addrNr);
						CString addrLastSeenLabel;
						addrLastSeenLabel.Format("fleet%u-unit%u-LastSeen", fleetNr, addrNr);
						CString addrCheckLabel;
						addrCheckLabel.Format("fleet%u-unit%u-Checked", fleetNr, addrNr);

						// create the unit keys
						WritePrivateProfileString(sysid, addrNameLabel, pAddr->GetItemText(), filename);
						WritePrivateProfileString(sysid, addrIdLabel, pAddr->GetSubItem(0), filename);
						WritePrivateProfileString(sysid, addrCommentLabel, pAddr->GetSubItem(1), filename);
						WritePrivateProfileString(sysid, addrLastSeenLabel, pAddr->GetSubItem(2), filename);
						WritePrivateProfileString(sysid, addrCheckLabel, pAddr->GetCheck()?"1":"0", filename);
						addrNr++;
					}
				}			
				fleetNr++;
			}
		}
	}

	return TRUE;
}

BOOL CAddrBookCtrl::LoadData(LPSTR filename)
{
	// get section names
	char names[4096];
	GetPrivateProfileSectionNames(names, 4096, filename);
    
	// loop sections
	char *sysid=names;
	while (sysid[0])
	{
		// get system info
		char sysname[128];
		GetPrivateProfileString(sysid, "sysName", "", sysname, 128, filename);
		char comment[512];
		GetPrivateProfileString(sysid, "sysComment", "", comment, 512, filename);

		// Find / Create system
		CTreeItem *pSystemNode = GetRoot(sysid);
		if (!pSystemNode)
		{
			CItemInfo *newItem = new CItemInfo();
			newItem->SetImage(4);
			newItem->SetItemText(sysname);
			newItem->SetSubItemText(0, sysid);
			newItem->SetControlType(newItem->constnum, 0);
			newItem->SetSubItemText(1, comment);
			pSystemNode = InsertRootItem(newItem);
			if (!pSystemNode)
			{
				MessageBox("Error inserting system in addressbook!");
				return FALSE;
			}
		}

		// read fleets for system
		UINT nrOfFleets = GetPrivateProfileInt(sysid, "numberOfFleets", 0, filename);
		UINT nrOfCheckedFleets = 0;
		for (UINT fleetNr=0; fleetNr<nrOfFleets; fleetNr++)
		{
			// make profile labels for this fleet
			CString fleetIdLabel;
			fleetIdLabel.Format("fleet%u-Id", fleetNr);
			CString fleetNameLabel;
			fleetNameLabel.Format("fleet%u-Name", fleetNr);
			CString fleetCommentLabel;
			fleetCommentLabel.Format("fleet%u-Comment", fleetNr);

			// get fleet info
			UINT fleet = GetPrivateProfileInt(sysid, fleetIdLabel, 0, filename);
			char name[128];
			GetPrivateProfileString(sysid, fleetNameLabel, "", name, 128, filename);
			char comment[512];
			GetPrivateProfileString(sysid, fleetCommentLabel, "", comment, 512, filename);
			
			// Create fleet
			if (fleet)
			{
				CString fleetid;
				fleetid.Format("%u", fleet);
				CTreeItem *pFleetNode = GetNode(pSystemNode, fleetid);
				if (!pFleetNode)
				{
					CItemInfo *newItem = new CItemInfo();
					newItem->SetImage(1);
					newItem->SetItemText(name);
					newItem->SetSubItemText(0, fleetid);
					newItem->SetControlType(newItem->constnum, 0);
					newItem->SetSubItemText(1, comment);
					newItem->SetCheck(TRUE);
					pFleetNode = InsertItem(pSystemNode, newItem);
					if (!pFleetNode)
					{
						MessageBox("Error inserting fleet in addressbook!");
						return FALSE;
					}					
				} else {
					MessageBox("Error in CAddrBookCtrl::LoadData(), duplicate fleet item in system!");
				}

				// read addresses for fleet
				CString fleetNrOfUnitsLabel;
				fleetNrOfUnitsLabel.Format("fleet%u-NrOfUnits", fleetNr);
				UINT nrOfAddrs = GetPrivateProfileInt(sysid, fleetNrOfUnitsLabel, 0, filename);
				UINT nrOfCheckedAddrs = 0;
				for (UINT addrNr=0; addrNr<nrOfAddrs; addrNr++)
				{
					// make profile labels for this fleet
					CString addrIdLabel;
					addrIdLabel.Format("fleet%u-unit%u-Id", fleetNr, addrNr);
					CString addrNameLabel;
					addrNameLabel.Format("fleet%u-unit%u-Name", fleetNr, addrNr);
					CString addrCommentLabel;
					addrCommentLabel.Format("fleet%u-unit%u-Comment", fleetNr, addrNr);
					CString addrLastSeenLabel;
					addrLastSeenLabel.Format("fleet%u-unit%u-LastSeen", fleetNr, addrNr);
					CString addrCheckLabel;
					addrCheckLabel.Format("fleet%u-unit%u-Checked", fleetNr, addrNr);

					// get fleet info
					UINT addr = GetPrivateProfileInt(sysid, addrIdLabel, 0, filename);
					UINT checked = GetPrivateProfileInt(sysid, addrCheckLabel, 0, filename);
					if (checked)
						nrOfCheckedAddrs++;
					char name[128];
					GetPrivateProfileString(sysid, addrNameLabel, "", name, 128, filename);
					char comment[512];
					GetPrivateProfileString(sysid, addrCommentLabel, "", comment, 512, filename);
					char lastseen[64];
					GetPrivateProfileString(sysid, addrLastSeenLabel, "", lastseen, 64, filename);

					// Find / Create addr
					if (addr)
					{
						CString addrid;
						addrid.Format("%u", addr);
						CTreeItem *pAddrNode = GetNode(pFleetNode, addrid);
						if (!pAddrNode)
						{
							CItemInfo *newItem = new CItemInfo();
							newItem->SetImage(2);
							newItem->SetItemText(name);
							newItem->SetSubItemText(0, addrid);
							newItem->SetControlType(newItem->constnum, 0);
							newItem->SetSubItemText(1, comment);
							newItem->SetSubItemText(2, lastseen);
							newItem->SetCheck(checked);
							if (!InsertItem(pFleetNode, newItem))
							{
								MessageBox("Error inserting addr in addressbook!");
								return FALSE;
							}
						} else {
							MessageBox("Error in CAddrBookCtrl::LoadData(), duplicate addr item in fleet!");
						}				
					}
				}
				// update checked status
				if (nrOfCheckedAddrs == 0)
					GetData(pFleetNode)->SetCheck(FALSE);
				else
					nrOfCheckedFleets++;
			}
		}
		// set checked status
		if (nrOfCheckedFleets > 0)
		{
			// first is for CItemInfo, second for CListView state
			GetData(pSystemNode)->SetCheck(TRUE);
			SetCheck(NodeToIndex(pSystemNode), TRUE);
		}

		// Sort the imported data
		Sort(pSystemNode, TRUE);

		// next system
		sysid += strlen(sysid)+1;
	}	
	return TRUE;
}


void CAddrBookCtrl::Initialize()
{
	LPTSTR lpszCols[] = {_T("Name"),_T("Ident"),_T("Comment"),_T("Last Seen"),0};
	LV_COLUMN   lvColumn;
	//initialize the columns
	lvColumn.mask = LVCF_FMT | LVCF_WIDTH | LVCF_TEXT | LVCF_SUBITEM;
	lvColumn.fmt = LVCFMT_LEFT;
	lvColumn.cx = 195;
	for(int x = 0; lpszCols[x]!=NULL; x++)
    {
		// set initial column widths
		switch(x)
		{
		case 1:
		  lvColumn.cx = 50;
		  break;
		case 3:
		  lvColumn.cx = 120;
		  break;
		default:
		  lvColumn.cx = 240;
		}

		lvColumn.pszText = lpszCols[x];
		InsertColumn(x,&lvColumn);
    }
}


void CAddrBookCtrl::AddUniqueAddr(snabel_addr_info_t a)
{
	CString system;
	system.Format("%u", a.system);
	CString fleet;
	fleet.Format("%u", a.fleet);
	CString addr;
	addr.Format("%u", a.addr);

	if (!a.system)
		return;
	// Find / Create system
	CTreeItem *pSystemNode = GetRoot(system);
	if (!pSystemNode)
	{
		CItemInfo *newItem = new CItemInfo();
		newItem->SetImage(4);
		newItem->SetItemText("New System");
		newItem->SetSubItemText(0, system);
		newItem->SetControlType(newItem->constnum, 0);
		newItem->SetSubItemText(1, "");
		pSystemNode = InsertRootItem(newItem);
		if (!pSystemNode)
		{
			MessageBox("Error inserting system in addressbook!");
			return;
		}
	}	

	if (!a.fleet)
		return;
	// Find / Create fleet
	CTreeItem *pFleetNode = GetNode(pSystemNode, fleet);
	if (!pFleetNode)
	{
		CItemInfo *newItem = new CItemInfo();
		newItem->SetImage(1);
		CString fleetname;
		fleetname.Format("Fleet %u", a.fleet);
		newItem->SetItemText(fleetname);
		newItem->SetSubItemText(0, fleet);
		newItem->SetControlType(newItem->constnum, 0);
		newItem->SetSubItemText(1, "");
		pFleetNode = InsertItem(pSystemNode, newItem);
		if (!pFleetNode)
		{
			MessageBox("Error inserting fleet in addressbook!");
			return;
		}
		// only sort if we're collapsed		
		if (IsCollapsed(pSystemNode))
			Sort(pSystemNode, FALSE);
	}

	if (!a.addr)
		return;
	// Find / Create addr
	CTreeItem *pAddrNode = GetNode(pFleetNode, addr);
	if (!pAddrNode)
	{
		CItemInfo *newItem = new CItemInfo();
		newItem->SetImage(2);
		newItem->SetItemText(addr);
		newItem->SetSubItemText(0, addr);
		newItem->SetControlType(newItem->constnum, 0);
		newItem->SetSubItemText(1, "");
		newItem->SetCheck(1);
		pAddrNode = InsertItem(pFleetNode, newItem);
		if (!pAddrNode)
		{
			MessageBox("Error inserting addr in addressbook!");
			return;
		}
		// only sort if we're collapsed		
		if (IsCollapsed(pFleetNode))
			Sort(pFleetNode, FALSE);
	}

	// set LastSeen time for this unit addr
	CTime now = CTime::GetCurrentTime();
	GetData(pAddrNode)->SetSubItemText(2, now.Format("%Y/%m/%d %H:%M:%S"));
}

CString CAddrBookCtrl::GetAddrName(snabel_addr_info_t a)
{
	CString name = "UNKNOWN";
	if (!a.system)
		return name;

	CString system;
	system.Format("%u", a.system);
	CString fleet;
	fleet.Format("%u", a.fleet);
	CString addr;
	addr.Format("%u", a.addr);

	name = system;
	CTreeItem *pSystemNode = GetRoot(system);
	if (pSystemNode)
	{
		if (a.fleet)
		{
			name = fleet;
			CTreeItem *pFleetNode = GetNode(pSystemNode, fleet);
			if (pFleetNode)
			{
				if (a.addr)
				{
					name = addr;
					CTreeItem *pAddrNode = GetNode(pFleetNode, addr);
					if (pAddrNode)
					{
						CItemInfo* pAddr = GetData(pAddrNode);
						if (pAddr)
						{
							name = pAddr->GetItemText();
						} else {
							MessageBox("Error getting addr node data in CAddrBookCtrl::GetAddrName()!");
						}
					}
				} else {
					CItemInfo* pFleet = GetData(pFleetNode);
					if (pFleet)
					{
						name = pFleet->GetItemText();
					} else {
						MessageBox("Error getting fleet node data in CAddrBookCtrl::GetAddrName()!");
					}
				}
			}
		} else {
			CItemInfo* pSystem = GetData(pSystemNode);
			if (pSystem)
			{
				name = pSystem->GetItemText();
			} else {
				MessageBox("Error getting system node data in CAddrBookCtrl::GetAddrName()!");
			}
		}
	}
		
	return name;
}

int CAddrBookCtrl::IsAddrChecked(snabel_addr_info_t a, BOOL select)
{
	CString system;
	system.Format("%u", a.system);
	CString fleet;
	fleet.Format("%u", a.fleet);
	CString addr;
	addr.Format("%u", a.addr);

	CTreeItem *pSystemNode = GetRoot(system);
	if (pSystemNode)
	{
		CTreeItem *pFleetNode = GetNode(pSystemNode, fleet);
		if (pFleetNode)
		{
			CTreeItem *pAddrNode = GetNode(pFleetNode, addr);
			if (pAddrNode)
			{
				CItemInfo* pAddr = GetData(pAddrNode);
				if (pAddr)
				{
					int retval = (pAddr->GetCheck()!=0);

					// Select this addr if it is checked
					if (select && retval)
						SelectNode(pAddrNode);

					// set LastSeen time for this unit addr
					CTime now = CTime::GetCurrentTime();
					pAddr->SetSubItemText(2, now.Format("%Y/%m/%d %H:%M:%S"));

					// return checked status (1==checked 0==notchecked)
					return retval;
				} else {
					MessageBox("Error getting addr node data in CAddrBookCtrl::IsAddrChecked()!");
					return FALSE;
				}
			}
		}
	}

	// address not found
	return -1;
}


CAddrBookCtrl::CTreeItem *CAddrBookCtrl::GetRoot(CString strItem)
{
	POSITION pos = GetRootHeadPosition();
	while(pos != NULL)
	{
		CTreeItem *pRoot = (CTreeItem*)GetNextRoot(pos);
		if (!pRoot)
		{
			MessageBox("Error in CAddrBookCtrl::GetRoot()!");
			return NULL;
		}
		CItemInfo *pItem = GetData(pRoot);
		if (!pItem)
		{
			MessageBox("Error in CAddrBookCtrl::GetRoot()!");
			return NULL;
		}
		if (pItem->GetSubItem(0) == strItem)
			return pRoot;
	}
	return NULL;
}

CAddrBookCtrl::CTreeItem* CAddrBookCtrl::GetNode(CTreeItem *pStartPosition, CString strItem)
{
	const UINT nChildren = NumChildren(pStartPosition);
	if (nChildren > 0)
	{
		POSITION pos = GetHeadPosition(pStartPosition);
		while (pos)
		{
			CTreeItem *pChild = GetNextChild(pStartPosition, pos);
			CItemInfo* lp = GetData(pChild);
			CString strData = lp->GetSubItem(0);
			if(strData==strItem)
				return pChild;
		}
	}
	return NULL;
}


//helper function to copy CItemInfo used when drag/drop you must override this this function to suit your own CItemInfo class
CItemInfo* CAddrBookCtrl::CopyData(CItemInfo* lpSrc)
{
	ASSERT(lpSrc!=NULL);
	CItemInfo* lpDest = new CItemInfo;
	//well okay I put all the copy thing in one function, located in CItemInfo class, 
	//so you should check out this function, remember to modify this function each time you add new data to CItemInfo class.
	lpDest->CopyObjects(lpSrc);
	return lpDest;
}


//override, like red!
COLORREF CAddrBookCtrl::GetCellRGB()
{
	return RGB(192,0,0);
}


//this is my override of GetIcon, override this to set what ever icon suits you
int CAddrBookCtrl::GetIcon(const CTreeItem* pItem)
{
	if(pItem!=NULL)
	{
		CItemInfo *pData = GetData(pItem);
		if (pData!=NULL)
		{
			int n = pData->GetImage();
			if(n!=-1)
				return n;
		
			int iImage = 0;
			if(ItemHasChildren(pItem))
			{
				IsCollapsed(pItem) ? iImage = 1/*close icon*/:iImage = 0;/*open icon*/
			}
			else
				iImage = 2;//doc icon
			return iImage;
		}
	}
	return 0;
}



//override
void CAddrBookCtrl::OnUpdateListViewItem(CTreeItem* lpItem, LV_ITEM *plvItem)
{
	//put some extra validation here 
	CString str = (CString)plvItem->pszText;
	if(!str.Compare(_T("Bugs: Impossible")))
	{
		str+=_T(" (selfconfidence allright)");//that's valid enough	:)
		CItemInfo *lp = GetData(lpItem);
		if(lp!=NULL)
		{
			if(plvItem->iSubItem==0)
				lp->SetItemText(str);
			else //subitem data 
				lp->SetSubItemText(plvItem->iSubItem-1, str);
		   UpdateData(lpItem, lp); //update internal nodes
		}
		SetItemText(plvItem->iItem, plvItem->iSubItem, str.GetBuffer(1));
	}
	else
		CSuperGridCtrl::OnUpdateListViewItem(lpItem, plvItem);
}



			
BOOL CAddrBookCtrl::OnItemLButtonDown(LVHITTESTINFO& ht)
{
	if(ht.iItem!=-1)
	{
#ifdef _WHAT_EVER_
		CTreeItem*pSelItem = GetTreeItem(ht.iItem);
		if(pSelItem!=NULL)
		{	
			CItemInfo* pInfo = GetData(pSelItem);
			if((ht.iSubItem==0) && (pInfo->GetItemText()==CString(_T("Hello World"))))
			{
				AfxMessageBox("no can do buddy");
				return 0;
			}

			for(int i=0;i<pInfo->GetItemCount(); i++)
			{
				if(i==ht.iSubItem-1)
				{
					CString str=pInfo->GetSubItem(i);
					if(str == CString(_T("Ethan Hunt"))) 
					{
						AfxMessageBox("Tom Cruise\nliked him in a few good men");
						return 0;
					}
				}
			}
		}
#endif
	}
	return 1;
}

//override called when OnLButtondown
void CAddrBookCtrl::OnControlLButtonDown(UINT nFlags, CPoint point, LVHITTESTINFO& ht)
{
	//now I am sure I added a combobox some where, so check for this control
	CTreeItem*pSelItem = GetTreeItem(ht.iItem);
	if(pSelItem!=NULL)
	{	
		CItemInfo* pInfo = GetData(pSelItem);
		CItemInfo::CONTROLTYPE ctrlType;
		if(pInfo->GetControlType(ht.iSubItem-1, ctrlType))
		{	
			if(ctrlType==pInfo->combobox) 
			{
					CStringList* list=NULL;
					pInfo->GetListData(ht.iSubItem-1, list);
					CComboBox * pList = ShowList(ht.iItem, ht.iSubItem, list);
			}

		} /*
		else //activate default edit control
			CSuperGridCtrl::OnControlLButtonDown(nFlags, point, ht);
			*/
	}
}



BOOL CAddrBookCtrl::OnVkReturn()
{
	BOOL bResult=FALSE;
	int iItem = GetNextItem( -1, LVNI_ALL | LVNI_SELECTED);
	if( GetCurSubItem() != -1 && iItem != -1)
	{
		CTreeItem*pSelItem = GetTreeItem(iItem);
		if(pSelItem!=NULL)
		{	
			CHeaderCtrl* pHeader = (CHeaderCtrl*)GetDlgItem(0);
			int iSubItem = Header_OrderToIndex(pHeader->m_hWnd, GetCurSubItem());
			CItemInfo* pInfo = GetData(pSelItem);
			CItemInfo::CONTROLTYPE ctrlType;
			if(pInfo->GetControlType(iSubItem-1, ctrlType))
			{	
				switch(ctrlType)
				{
					case pInfo->constnum:
						bResult=TRUE; // do nothing
						break;

					case pInfo->combobox: 
						{

							CStringList* list=NULL;
							pInfo->GetListData(iSubItem-1, list);
							CComboBox * pList = ShowList(iItem, iSubItem, list);
							bResult=TRUE; //I'll handle it from here
						}break;
					default:break;
				}
			}
		}
	}
	return bResult;
}





#define IDC_COMBOBOXINLISTVIEW 0x1235
CComboBox* CAddrBookCtrl::ShowList(int nItem, int nCol, CStringList *lstItems)
{
	CString strFind = GetItemText(nItem, nCol);

	//basic code start
	CRect rect;
	int offset = 0;
	// Make sure that the item is visible
	if( !EnsureVisible(nItem, TRUE)) return NULL;
	GetSubItemRect(nItem, nCol, LVIR_BOUNDS, rect);
	// Now scroll if we need to expose the column
	CRect rcClient;
	GetClientRect(rcClient);
	if( offset + rect.left < 0 || offset + rect.left > rcClient.right )
	{
		CSize size;
		size.cx = offset + rect.left;
		size.cy = 0;
		Scroll(size);
		rect.left -= size.cx;
	}
	
	rect.left += offset;	
	rect.right = rect.left + GetColumnWidth(nCol);
	if(rect.right > rcClient.right) 
	   rect.right = rcClient.right;
	//basic code end

	rect.bottom += 10 * rect.Height();//dropdown area
	
	DWORD dwStyle =  WS_CHILD | WS_VISIBLE | WS_VSCROLL | WS_HSCROLL|CBS_DROPDOWNLIST | CBS_DISABLENOSCROLL;
	CComboBox *pList = new CComboInListView(nItem, nCol, lstItems);
	pList->Create(dwStyle, rect, this, IDC_COMBOBOXINLISTVIEW);
	pList->ModifyStyleEx(0,WS_EX_CLIENTEDGE);//can we tell at all
	pList->SetHorizontalExtent(CalcHorzExtent(pList, lstItems));
	pList->ShowDropDown();
	pList->SelectString(-1, strFind.GetBuffer(1));
	// The returned pointer should not be saved
	return pList;
}



int CAddrBookCtrl::CalcHorzExtent(CWnd* pWnd, CStringList *pList)
{
	int nExtent=0;
	if(pWnd!=NULL)
	{
		CDC* pDC = pWnd->GetDC();
		HFONT hFont = (HFONT)pWnd->SendMessage(WM_GETFONT); //why not pWnd->GetFont();..I like the send thing alot and
		CFont *pFont = CFont::FromHandle(hFont);			//this way I get to use this function..cool :)
		if(pFont!=NULL)										//ya what ever makes me happy,.right :}
		{
			CFont* pOldFont = pDC->SelectObject(pFont);
			CSize newExtent;
			POSITION pos = pList->GetHeadPosition();
			while(pos != NULL)
			{
				CString str(pList->GetNext(pos));
				newExtent = pDC->GetTextExtent(str);
				newExtent.cx += 6;
				if (newExtent.cx > nExtent)
				{
					nExtent = newExtent.cx;
				}
			}
			pDC->SelectObject(pOldFont);
		}
		pWnd->ReleaseDC(pDC);
	}
	return nExtent;
}



void CAddrBookCtrl::SortData()
{
	int nIndex = GetNextItem(-1, LVNI_ALL | LVNI_SELECTED); 
	if(nIndex==-1)
		return;

	CTreeItem *pItem = reinterpret_cast<CTreeItem*>(GetItemData(nIndex));

	if(AfxMessageBox("Sort all children(Yes)\nor just sort rootitems(No)",MB_YESNO)==IDYES)
		Sort(pItem, TRUE);		
	else
		Sort(pItem, FALSE);
	//do a simple refresh thing
	if(ItemHasChildren(pItem))
	{
		SetRedraw(0);
		Collapse(pItem);
		Expand(pItem, nIndex);
		SetRedraw(1);
	}
}



BOOL CAddrBookCtrl::OnDeleteItem(CTreeItem* pItem, int nIndex)
{
	return 1;
}



BOOL CAddrBookCtrl::OnItemExpanding(CTreeItem *pItem, int iItem)
{
	return 1;
}


BOOL CAddrBookCtrl::OnItemExpanded(CTreeItem* pItem, int iItem)
{
	return 1;
}


BOOL CAddrBookCtrl::OnCollapsing(CTreeItem *pItem)
{
	return 1;
}



BOOL CAddrBookCtrl::OnItemCollapsed(CTreeItem *pItem)
{
	Sort(pItem, FALSE);
	return 1;
}


CImageList *CAddrBookCtrl::CreateDragImageEx(int nItem)
{
		if(m_bDrag)
			return CSuperGridCtrl::CreateDragImageEx(GetDragItem());
		else
			return NULL;
}
