/* $Id: FSK.h,v 1.2 2007/01/15 21:47:00 olle Exp $
 *
 * The code for this demodulator was more or less stolen directly from 
 * spe@stacken.kth.se. It originated in his "ape" project, so here goes:
 *
 * Copyright (c) 2003, Stefan Petersen
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * 3. Neither the name of the author nor the names of any contributors
 *    may be used to endorse or promote products derived from this 
 *    software without specific prior written permission.
 *   
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 * 
 */

#ifndef _FSK_H
#define _FSK_H

#include "PLL.h"

#include <math.h>

#define M_PI   3.14159265358979323846 

class CFSKDemodulator
	: public CPhaseLockedLoop
{
private:
  const long m_kSamplesPerSec;
  const long m_kBPS;

  bool m_bInvert;

  double *i0, *q0, *i1, *q1;
  double *corr_i0, *corr_q0, *corr_i1, *corr_q1;
  double sum[4];
  unsigned int curr_sample, samplespersym, ones_calc_mask, freq0, freq1, corr, ones, pll;

  virtual void handleBit(bool bit) = 0;

public:
  CFSKDemodulator(const long nSamplesPerSec, const long nBPS, unsigned short f0, unsigned short f1) :
	m_kSamplesPerSec(nSamplesPerSec),
    m_kBPS(nBPS),

	m_bInvert(false),

	curr_sample(0),
	ones(0),
	corr(0),
	freq0(f0),
	freq1(f1)
  {
      pll = samplespersym = (int)floor((float)nSamplesPerSec/nBPS);
	  ones_calc_mask = (1<<(samplespersym - 1));

      i0 = new double[samplespersym * 2];
      q0 = new double[samplespersym * 2];
      i1 = new double[samplespersym * 2];
      q1 = new double[samplespersym * 2];
    
      corr_i0 = new double[samplespersym];
      corr_q0 = new double[samplespersym];
      corr_i1 = new double[samplespersym];
      corr_q1 = new double[samplespersym];

      memset((void *)corr_i0, 0, samplespersym * sizeof(double));
      memset((void *)corr_q0, 0, samplespersym * sizeof(double));
      memset((void *)corr_i1, 0, samplespersym * sizeof(double));
      memset((void *)corr_q1, 0, samplespersym * sizeof(double));
	  memset((void *)sum, 0, 4 * sizeof(double));

      for (unsigned int i = 0; i < samplespersym * 2 ; i++)
	  {
      i0[i] = sin(2.0*M_PI*((double)i/(double)samplespersym) * ((double)freq0/(double)m_kBPS));
      q0[i] = cos(2.0*M_PI*((double)i/(double)samplespersym) * ((double)freq0/(double)m_kBPS));
      i1[i] = sin(2.0*M_PI*((double)i/(double)samplespersym) * ((double)freq1/(double)m_kBPS));
      q1[i] = cos(2.0*M_PI*((double)i/(double)samplespersym) * ((double)freq1/(double)m_kBPS));
      }
	  
  };
  virtual ~CFSKDemodulator()
  {
	  delete [] i0;
	  delete [] q0;
	  delete [] i1;
	  delete [] q1;

	  delete [] corr_i0;
	  delete [] corr_q0;
	  delete [] corr_i1;
	  delete [] corr_q1;
  };

  void setInvert(bool bInvert) { m_bInvert = bInvert; };
  bool getInvert(void) const { return m_bInvert; };

  void inputData(unsigned short* buffer, unsigned int nSamples)
  {
	  for (unsigned int i = 0; i < nSamples; ++i) {
		double sample = ((double)buffer[i]-32768.0)/32768.0;

		// discard old data
	    sum[0] -= corr_i0[curr_sample%samplespersym];
	    sum[1] -= corr_q0[curr_sample%samplespersym];
	    sum[2] -= corr_i1[curr_sample%samplespersym];
	    sum[3] -= corr_q1[curr_sample%samplespersym];
	    
		// calculate this sample
	    corr_i0[curr_sample%samplespersym] = i0[curr_sample] * sample;
	    corr_q0[curr_sample%samplespersym] = q0[curr_sample] * sample;
	    corr_i1[curr_sample%samplespersym] = i1[curr_sample] * sample;
	    corr_q1[curr_sample%samplespersym] = q1[curr_sample] * sample;
	    
		// accumulate data
	    sum[0] += corr_i0[curr_sample%samplespersym];
	    sum[1] += corr_q0[curr_sample%samplespersym];
	    sum[2] += corr_i1[curr_sample%samplespersym];
	    sum[3] += corr_q1[curr_sample%samplespersym];
	    
	    curr_sample++;
	    curr_sample %= (samplespersym * 2);

		// correlate	    
	    if (corr & ones_calc_mask)
			--ones;
	    corr <<= 1;
		double phase = sqrt(sum[2]*sum[2] + sum[3]*sum[3]) - sqrt(sum[0]*sum[0] + sum[1]*sum[1]);
	    if (phase > 0.0){
			++corr;
			++ones;
	    }
	    
	    if (--pll == 0)
		{
			// deliver bit
		    if (ones > 4 && !m_bInvert)
				handleBit(1);
			else 
				handleBit(0);

			// early-late gate synchronization
			int diff = 0;
			if (ones > 4) {
				if ((corr & 0x0028) != 0x0028) {
					if (corr & 0x0008)
						diff = 3;
					else
						diff = -3;
				} else if ((corr & 0x0044) != 0x0044) {
					if (corr & 0x0004)
						diff = 2;
					else
						diff = -2;
				} else if ((corr & 0x0082) != 0x0082) {
					if (corr & 0x0002)
						diff = 1;
					else
						diff = -1;
				} else if ((corr & 0x0101) != 0x0101) {
					diff = 0;
				} else {
					diff = 0;				
				}
			}
			pll = samplespersym + diff;
		}
	  }
  };
};

#endif