/* $Id: GMSK.h,v 1.1 2007/01/11 16:57:04 olle Exp $ */

#ifndef _GMSK_H
#define _GMSK_H

#include "PLL.h"

class CGMSKDemodulator
	: CPhaseLockedLoop
{
private:
  const long m_kSamplesPerSec;
  const long m_kBPS;
  const double m_kBitTime;
  const double m_kBitHalfTime;

  long m_nAccumulate;

  double m_fAccumulate;
  double m_fPhase;

protected:
   // must be provided by subclass
   virtual void handleBit(bool bDataBit) = 0;

public:
  CGMSKDemodulator(const long nSamplesPerSec, const long nBPS) :
	m_kSamplesPerSec(nSamplesPerSec),
    m_kBPS(nBPS),
    m_kBitTime(1.0 / nBPS),
    m_kBitHalfTime(0.5 / nBPS),

	m_nAccumulate(0),
    m_fAccumulate(0.0),
    m_fPhase(m_kBitHalfTime)
  {
  };
  virtual ~CGMSKDemodulator() {};

  void inputData(unsigned short* buffer, unsigned int nSamples)
  {
	unsigned nDex = 0;
	long nCount = 0;
	signed short* lpData = (signed short*)buffer;
	while (nDex < nSamples)
	{
		bool bBit = lpData[nDex++] < 0;
		nCount++;
		while ( nDex < nSamples && bBit == (bool)(lpData[nDex] < 0) )
		{
			nDex++;
			nCount++;
		}
		double fDelta = nCount;
		fDelta /= m_kSamplesPerSec;
		m_fAccumulate += fDelta;
		while ( m_fAccumulate >= m_kBitHalfTime)
		{
			handleBit(bBit);
			m_fAccumulate -= m_kBitTime;
		}
		m_fPhase = m_fAccumulate;
		nCount = 0;
	}   
  };
};

#endif
