/* $Id: MPT1327.cpp,v 1.4 2007/01/21 23:10:41 olle Exp $ */

#include "stdafx.h"

#include <stdlib.h>
#include <stdio.h>
#include <time.h>

#include "MPT1327.h"

#include "cmdWndProt.h"
#include "callinfo.h"

#define MPT1327_POLYNOM 0x6815

void CMPT1327::pprint (char *msg)
{
	if (cmdWnd && strlen(msg))
		CMDWND_ADD_LOGSTR(cmdWnd, msg);
	ptext[0]=0;
}

void CMPT1327::reportAddr(unsigned short prefix, unsigned short ident)
{
	if (ident >= 8185) // skip system IDs
		return;

	// report new addresses
	snabel_addr_info_t *addr = new snabel_addr_info_t;					
	addr->system = currentsystem;
	addr->fleet = prefix;
	addr->addr = ident;
	CMDWND_REPORT_ADDR(cmdWnd, addr);
}

int CMPT1327::getbit(int n, unsigned short *codewords)
{
    unsigned short mask;
    unsigned short codeword;

    codeword = codewords[n/16];
    mask = 1<<(15-(n%16));

    return (codeword & mask)?1:0;
}


unsigned short CMPT1327::fcs()
{
    int n, bit;
    unsigned short syndrome = 0, parity = 0;

    for (n = 0; n < 64; n++) {
	bit = getbit(n, codeword);
	parity ^= bit;
	if (n == 62) bit ^= 1;
	if (n < 63) {
	    syndrome <<= 1;
	    if (bit ^ ((syndrome & 0x8000)?1:0))
		syndrome ^= 0x6815;
	}
    }

    syndrome &= 0x7fff;
    if (parity)
	syndrome |= 0x8000;
    
    return syndrome;
}

void CMPT1327::handleBit(bool bit)
{
    unsigned short channel;
    unsigned short channelc;
    unsigned short prefix = 0;        
    unsigned short prefix2;
    unsigned short ident1 = 0;
    unsigned short ident2;
    unsigned short wt;
    unsigned short rsvd;
	unsigned short status;
    unsigned short alm;
    unsigned short aln;
    unsigned short sysdef;
    unsigned short sysid;
    unsigned short cat;
    unsigned short type;
    unsigned short func;

	if (!cmdWnd)
		MessageBox(NULL, "No cmdWnd parameter passed to mpt1327 decoder!", NULL, MB_OK);	

    /*
     * Shift in the least significant bit into the 64 bit frame.
     */
    codeword[0] <<= 1;
    codeword[0] += (codeword[1] & 0x8000)?1:0;
    codeword[1] <<= 1;
    codeword[1] += (codeword[2] & 0x8000)?1:0;
    codeword[2] <<= 1;
    codeword[2] += (codeword[3] & 0x8000)?1:0;
    codeword[3] <<= 1;
    if (bit) codeword[3] += 1;

	// update bitcounter
	if (cnt>0)
		cnt--;

	// if not synched
	if (cnt==0)
	{
		// check for synchronization sequence
		if  (((codeword[2] & 0xffff) == 0xaaaa && (codeword[3] & 0xffff) == MPT1327_SYNC) ||
			 ((codeword[2] & 0xffff) == 0xaaaa && (codeword[3] & 0xffff) == MPT1327_SYNT))
		{
			// report found sync
			CMDWND_GOT_SYNC(cmdWnd);

			// reset sync timer
			synctimer = 1;

			// paritycheck the codeword (is this a valid CCSC/DSCS?)
			if (fcs() == 0)
			{
				// detect system site changes
				if (currentsysid!=(codeword[0] & 0x7fff) && (codeword[0] & 0x7fff)!=0)
				{
					// save current SYSID					
					currentsysid=codeword[0] & 0x7fff;

					// parse SYSID into system and site		
					if (codeword[0] & 0x8000)
					{
						// National network
						currentsystem = ((currentsysid >> 12) & 0x3) + 1000;
						currentsite = (currentsysid >> 3) & 0x1ff;
					} else {
						// Regional network
						currentsystem = (currentsysid >> 7) & 0x7f;
						currentsite = (currentsysid >> 3) & 0xf;
					}
					
					// translate system 0 to non-zero value
					if (0 == currentsystem)
						currentsystem = 0x80;

					// report new system
					reportAddr(0,0);

					if (((codeword[2]) == 0xaaaa)&&((codeword[3]) == 0xc4d7)&&(!((codeword[0]) >>15 & 0x1))){
						sprintf(ptext,"CCSC | Control Channel System Codeword | SYSId: 0x%4.4x NET: %u SITE: %u LAB: %u", currentsysid, currentsystem, currentsite, currentsysid & 0x7);
						pprint(ptext);
					}

					if (((codeword[2]) == 0xaaaa)&&((codeword[3]) == 0x3b28)&&(!((codeword[0]) >>15 & 0x1))){
						sprintf(ptext,"DCSC | Data Channel Systm Codeword     | SYSId: 0x%4.4x NET: %u SITE: %u LAB: %u", currentsysid, currentsystem, currentsite, currentsysid & 0x7);
						pprint(ptext);
					}        
				}
			}

			// set number of bits til next codeword
			cnt = 64+1;

		} else {
			// if synched, tick the synctimer
			if(synctimer && synctimer++ > 192)
			{
				// report lost sync after 192 ticks
				CMDWND_LOST_SYNC(cmdWnd);
				synctimer = 0;
			}
		}
	}

	// codeword received
	if (cnt == 1)
	{
#ifdef DEBUG_CW
		sprintf(ptext,"DEBUG| Codeword %.4x %.4x %.4x %.4x    |", codeword[0], codeword[1], codeword[2], codeword[3]);
		pprint(ptext);
#endif
		// paritycheck the codeword
		if (fcs() == 0)
		{
			// reset sync timer
			synctimer = 1;

			// process the correctly received codeword
			if ((codeword[0] & 0x8000) == 0)
			{
				if  (((codeword[2] & 0xffff) == 0xaaaa && (codeword[3] & 0xffff) == MPT1327_SYNC) ||
					 ((codeword[2] & 0xffff) == 0xaaaa && (codeword[3] & 0xffff) == MPT1327_SYNT))
				{
					// CCSC/DCSC handled in sync routine
				} else {

					// Data Codeword
					// TODO: dump codeword data
					sprintf(ptext,"DATA | Unhandled Data Codeword         | %.4x %.4x %.4x %.4x", codeword[0], codeword[1], codeword[2], codeword[3]);
					pprint(ptext);
				}
				
			} else { // Address Codeword

				// GTC - call setup
				if ((codeword[1] & (1<<10)) == 0) {
					
					snabel_call_info_t* call = new snabel_call_info_t;
					strcpy(ptext,"GTC  | Go To Channel ");
					if (codeword[1] & 0x200) {
						call->calltype = SNABEL_CALL_TYPE_DATA_DGRAM;
						strcat(ptext,"Data              | ");                                   /* Data Call */
					} else {
						call->calltype = SNABEL_CALL_TYPE_ANALOG_VOICE;
						strcat(ptext,"Voice             | ");                                   /* Voice Call */				  
					}

					// parse call parameters
					prefix = ((codeword[0] & 0x7f00) >> 8);
					call->ctrlchnl = currentchannel;
					call->calladdr.system = call->fromaddr.system = currentsystem;
					call->calladdr.fleet = call->fromaddr.fleet = prefix;
					call->calladdr.addr = ident1 = ((codeword[0] << 5 | codeword[1] >> 11) & 0x1fff);
					call->fromaddr.addr = ident2 = ((codeword[2] >>2) & 0x1fff);
					call->callchnl = channel = ( (codeword[1] << 1 | codeword[2] >> 15) & 0x3ff);

					// report new call
					CMDWND_REPORT_CALL(cmdWnd, call);

					sprintf(ptext,"%sCh: %3d (%3d) Org: %4d Tar: %4d ", ptext, channel, prefix, ident2, ident1);
					/*if (!(codeword[1] & 0x200)) {
						time_t t = time(NULL);                        
						sprintf (ptext,"%s%s", ptext, ctime(&t));                        
					}*/

				} else { // not GTC - General Address Codeword Structure

					// get cat/type/func and parse the codeword
                cat = ((codeword[1]>>7) & 0x7);
                type = ((codeword[1]>>5) & 0x3);
                func = ((codeword[1]>>2) & 0x7);

				switch (cat)
				{
				// Category '000' Messages
				case 0x0:
					switch (type)
					{
					// Category '000' Type '00' : Aloha invitations
					case 0x0:
						switch (func)
						{
						case 0x0:
#ifdef DEBUG
							strcat(ptext,"ALH  | General single Codeword         |");
#endif
							break;
						case 0x1:
							strcat(ptext,"ALHS | Messages invited, except RQD    |");
							break;
						case 0x2:
							strcat(ptext,"ALHD | Messages invited, except RQS    |");
							break;
						case 0x3:
							strcat(ptext,"ALHE | Emergency requests (RQE) only   |");
							break;
						case 0x4:                    
							strcat(ptext,"ALHR | Registration/Emergency request  |");
							break;
						case 0x5:                    
							strcat(ptext,"ALHX | Messages invited, except RQR    |");
							break;
						case 0x6:                    
							strcat(ptext,"ALHF | Fall-back mode                  |");
							break;
						case 0x7:                    
							strcat(ptext,"ALH? | Reserved                        |");
							break;
						}

						prefix = ((codeword[0] & 0x7f00) >> 8);
						ident1 = ((codeword[0] << 5 | codeword[1] >> 11) & 0x1fff);

						// report address
						//report_addr(cmdWnd, prefix, ident1);

						wt = ((codeword[2] >> 11) & 0x7) ;
						rsvd = ((codeword[2] >> 9) & 0x3) ;
						alm = ((codeword[2] >> 4) & 0x1f) ;
						aln = ((codeword[2]) & 0xf) ;
						if (strlen(ptext))
							sprintf(ptext,"%s (%3d) Id: %4d WT:%d RSVD:%d M:%d N:%d",ptext,prefix,ident1,wt,rsvd,alm,aln);                    
						break;
                                                      
					// Category '000' Type '01' : Acknowledgements
					case 0x1:
						switch(func)
						{
						case 0x0:
							strcat(ptext,"ACKK | General acknowledgement         |");
							break;
						case 0x1:
							strcat(ptext,"ACKI | Intermediate ack, more follows  |");
							break;
						case 0x2:
							strcat(ptext,"ACKQ | Acknowledge, call queued        |");
							break;
						case 0x3:
							strcat(ptext,"ACKX | Acknowledge, message rejected   |");
							break;
						case 0x4:
							strcat(ptext,"ACKV | Acknowledge, unit unavailable   |");
							break;
						case 0x5:
							strcat(ptext,"ACKE | * Acknowledge, emergency call * |");
							break;
						case 0x6:
							strcat(ptext,"ACKT | Acknowledge, try given address  |");
							break;
						case 0x7:						
							strcat(ptext,"ACKB |Acknowledge, call-back or negat. |");
							break;
						default:
							sprintf(ptext, "ACK? | Unknown                        | CAT: 0x%1.1x TYPE: 0x%1.1x FUNC: 0x%1.1x", cat, type, func);
							break;
						}
	                    prefix = ((codeword[0] & 0x7f00) >> 8);
		                ident1 = ((codeword[0] << 5 | codeword[1] >> 11) & 0x1fff);
			            ident2 = ((codeword[1] << 11 | codeword[2] >> 5) & 0x1fff);

						// report new addresses
						reportAddr(prefix, ident1);
						reportAddr(prefix, ident2);

						sprintf(ptext,"%s (%3d) Org: %4d Tar: %4d",ptext,prefix,ident2,ident1);
						break;

					// Category '000' Type '10' : Requests/Ahoys
					case 0x2:
	                    prefix = ((codeword[0] & 0x7f00) >> 8);
		                ident1 = ((codeword[0] << 5 | codeword[1] >> 11) & 0x1fff);
			            ident2 = ((codeword[1] << 11 | codeword[2] >> 5) & 0x1fff);
						status = (codeword[2] & 0x1f);
				        rsvd = ((codeword[2] >> 3) & 0x3); // RQC slots

						// report new addresses
						reportAddr(prefix, ident1);
						reportAddr(prefix, ident2);

						switch (func)
						{
						case 0x0:
							sprintf(ptext,"RQS  | Request random access           | (%3d) Org: %4d Tar: %4d",prefix,ident2,ident1);
							if ((codeword[2]) >>4 & 0x1){  // DT
								strcat(ptext," Voice");
				            }else{
								strcat(ptext," Data");
							}
		                    if (!(codeword[2]) >>3 & 0x1) strcat(ptext," Priority");
							pprint(ptext);

				            strcpy(ptext, "AHY  | Availability check message      | Begin Ack a: ");
		                    if ((codeword[2]) >>3 & 0x1){ // point
				            sprintf(ptext,"%s(%3d) Org: %4d",ptext,prefix,ident2);
							}else{
							sprintf(ptext,"%s(%3d) Tar: %4d",ptext,prefix,ident1);
							}
				            if ((codeword[2]) >>4 & 0x1){  // D
		                      strcat(ptext," Voice");
				            }else{
		                      strcat(ptext," Data");
							}
							    if ((codeword[2]) >>1 & 0x1)
									strcat(ptext," Emergency");
							break;
                    
						case 0x1:
							strcpy(ptext, "RQ   | REQ dedicated                   |");
							break;
                    
						case 0x2:
							sprintf(ptext,"RQX  | Request call cancel/abort       | (%3d) Org: %4d Tar: %4d",prefix,ident2,ident1);
							pprint(ptext);
							sprintf(ptext,"AHYX | Cancel alert/waiting state      | (%3d) Org: %4d Tar: %4d",prefix,ident2,ident1);
							break;

						case 0x3:
							strcpy(ptext,"RQT  |");
							if ((codeword[2]) >>2 & 0x1){  //DIV
								strcat(ptext," Cancel");
							}else{
								strcat(ptext,"Request");
							}
							sprintf(ptext,"%s Request call diversion    | (%3d) Org: %4d Tar: %4d",ptext,prefix,ident2,ident1);
							if ((codeword[2]) >>4 & 0x1){  // DT
								strcat(ptext," Voice");
							}else{
								strcat(ptext," Data");
							}
							if (!(codeword[2]) >>3 & 0x1) strcat(ptext," Vorrang");
							break;

						case 0x4:
							sprintf(ptext,"RQE  | Request emergency call          | (%3d) Org: %4d Tar: %4d ",prefix,ident2,ident1);
							if ((codeword[2]) >>4 & 0x1){  // DT
								strcat(ptext," Voice");
							}else{
								strcat(ptext," Data");
							}
							break;

						case 0x5:
							sprintf(ptext,"RQR  | Request to register             | (%3d) Org: %4d",prefix,ident1);
							pprint(ptext);
							sprintf(ptext,"AHYP | Called Unit Presence Monitoring | (%3d) Org: %4d Tar: %4d",prefix,ident2,ident1);
							break;

						case 0x6:
							sprintf(ptext,"RQQ  | Request status transaction      | (%3d) Org: %4d Tar: %4d Status: %2u",prefix,ident2,ident1,status);
							pprint(ptext);
							sprintf(ptext,"AHYQ | Status message                  | (%3d) Org: %4d Tar: %4d Status: %2u",prefix,ident2,ident1,status);
							break;

						case 0x7:
							sprintf(ptext,"RQC  | Request to send short data mess | (%3d) Org: %4d Tar: %4d Slots:%d",prefix,ident2,ident1,rsvd);
							pprint(ptext);
							sprintf(ptext,"AHYC | Short data invitation message   | (%3d) Org: %4d Tar: %4d Slots:%d",prefix,ident2,ident1,rsvd);
							break;

						default:
							sprintf(ptext,"RQT  | Unknown                         | CAT: 0x%1.1x TYPE: 0x%1.1x FUNC: 0x%1.1x", cat, type, func);
							break;
						}
						break;
						
					// Category '000' Type '11' : Miscellaneous
					case 0x3:
		                channel = ((codeword[1] << 8 | codeword[2] >> 8) & 0x3ff);
						switch(func)
						{
						case 0x0:
							sprintf (ptext,"MARK | Control channel marker          | (Act. Chan: %3d)", channel);
							currentchannel = channel;
							break;

						case 0x1:
						    sprintf (ptext,"MAINT| Call maintenance message        | (Act. Chan: %3d)", channel);
							currentchannel = channel;
							break;

						case 0x2:
							{
								//time_t t = time(NULL);
								channel = (codeword[0]>>5 & 0x2ff);
								channelc = ((codeword[0] << 5 | codeword[1] >> 11) & 0x3ff);
							
								snabel_call_info_t* call = new snabel_call_info_t;
								call->calladdr.system = currentsystem;
								call->callchnl = channel;
								call->ctrlchnl = channelc;
								CMDWND_CLEAR_CALL(cmdWnd, call);
														
								sprintf(ptext,"CLEAR| Call clear-down                 | Ch: %3d -> CCh: %3d", channel,channelc/*, ctime(&t)*/);
							}
							break;

						case 0x3:
							sprintf (ptext, "MOVE | Move to specified ControlChannel | -> CCh: %3d",channel);
							break;

						case 0x4:
		                   sysdef=((codeword[0] & 0x7c00) >> 10);
				           sysid=((codeword[0] << 5 | codeword[1] >> 11) & 0x7fff);                  
						   switch (sysdef)
						   {
						   case 0x0:							
							   sprintf(ptext,"BCAST| Announce Control Channel        | SYSId: 0x%4.4x CCh: %3d",sysid, ((codeword[1] << 8 | codeword[2] >> 8) & 0x3ff));
							   break;

						   case 0x1:                   
							   sprintf(ptext,"BCAST| Withdraw Control Channel        | SYSId: 0x%4.4x CCh: %3d",sysid, ( (codeword[1] << 8 | codeword[2] >> 8) & 0x3ff));
							   break;
                   
						   case 0x2:
#ifdef DEBUG
							   sprintf(ptext,"BCAST| Call maintenance parameters     | SYSId: 0x%4.4x  SYSDEF: 0x%2.2x",sysid, sysdef);                                      
#endif
							   break;
                   
						   case 0x3:
#ifdef DEBUG
							   sprintf(ptext,"BCAST| Registration Parameters         | SYSId: 0x%4.4x SYSDEF: 0x%2.2x",sysid, sysdef);
#endif
							   break;
                   
						   case 0x4:                   
							   sprintf(ptext,"BCAST| Announce Adjacent Site          | SYSId: 0x%4.4x Ch: %3d  Id red: %d",sysid, ( (codeword[1] << 8 | codeword[2] >> 8) & 0x3ff),(codeword[2] & 0xf));
							   break;

						   case 0x5:                   
							   sprintf(ptext,"BCAST| Vote Now advice                 | SYSId: 0x%4.4x Ch: %3d  Id red: %d ",sysid, ((codeword[1] << 8 | codeword[2] >> 8) & 0x3ff),(codeword[2] & 0xf));                   
							   break;

						   default:
							   sprintf(ptext,"BCAST| Reserved                        | SYSId: 0x%4.4x SYSDEF: 0x%2.2x",sysid, sysdef);
							   break;
						   }        
						   break;

						default:
							sprintf(ptext, "MISC | Reserved                        | CAT: 0x%1.1x TYPE: 0x%1.1x FUNC: 0x%1.1x", cat, type, func);
							break;
						}
						break;

					default:
						// Unknown type for Category '000'
						sprintf(ptext, "UNKNOWN Codeword  CAT: 0x%1.1x TYPE: 0x%1.1x FUNC: 0x%1.1x", cat, type, func);
						break;
				  }
				  break;

				// Category '001' Messages : TODO
				case 0x1:
					switch(type)
					{
					// Category '001' Type '00' : 
					case 0x0:
						prefix = ((codeword[0] & 0x7f00) >> 8);
						ident1 = ((codeword[0] << 5 | codeword[1] >> 11) & 0x1fff);

						// report new address
						reportAddr(prefix, ident1);

						sprintf(ptext,"SAM  | Single address messages          | Tar: (%3d)%4d",prefix,ident1);
						break;

					// Category '001' Type '01' : 
					case 0x1:
			            prefix = ((codeword[0] & 0x7f00) >> 8);
						ident1 = ((codeword[0] << 5 | codeword[1] >> 11) & 0x1fff);
						prefix2 = ((codeword[1] | codeword[2] >> 13) & 0x7f);
						ident2 = ((codeword[2]) & 0x1fff);

						// report new addresses
						reportAddr(prefix, ident1);
						reportAddr(prefix2, ident2);

						rsvd = ((codeword[1]) >>4 & 0x3);  // len
						sprintf(ptext,"HEAD | Short data message               | Org:(%3d)%4d Tar:(%3d)%4d  Cwords: %d",prefix,ident2,prefix2,ident1,rsvd);
						break;

					default:
						// Unknown type for Category '001'
						sprintf(ptext, "UNKNOWN Codeword  CAT: 0x%1.1x TYPE: 0x%1.1x FUNC: 0x%1.1x", cat, type, func);
						break;
	                }
					break;

				default:
					// Unknown Category
					sprintf(ptext, "UNKNOWN Codeword  CAT: 0x%1.1x TYPE: 0x%1.1x FUNC: 0x%1.1x", cat, type, func);
					break;
				}
               
				} // GTC / not GTC
				pprint(ptext);

			} // Address Codeword

			// set number of bits til next codeword
			cnt = 64+1;

		} else {
			// mysterious codeword, assume it is the last codeword before "Hang-Over Bit"
			if (codeword[0]==0x8000 && (codeword[1]&0xfffe)==0x0200)
			{
				cnt = 1;
			} else {
#ifdef DEBUG
				sprintf(ptext,"ERROR| Codeword parity check failed.   | %.4x %.4x %.4x %.4x", codeword[0], codeword[1], codeword[2], codeword[3]);
				pprint(ptext);
#endif
				// lost sync
				cnt = 0;
			}
		}

	} // codeword received
}
