/* $Id: MixerCtrl.cpp,v 1.8 2006/09/22 17:02:04 olle Exp $
 *
 * Copyright (c) 2006, olle@toolcrypt.org
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 *    * Redistributions of source code must retain the above copyright notice,
 *      this list of conditions and the following disclaimer.
 *    * Redistributions in binary form must reproduce the above copyright notice,
 *      this list of conditions and the following disclaimer in the documentation
 *      and/or other materials provided with the distribution.
 *    * Neither the name of the <ORGANIZATION> nor the names of its contributors may
 *      be used to endorse or promote products derived from this software without 
 *      specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include "stdafx.h"
#include "MixerCtrl.h"


CMixerCtrl::CMixerCtrl()
{
	MMRESULT result = mixerOpen(&m_hmixer, MIXER_OBJECTF_MIXER, 0, 0, 0);
	if (MMSYSERR_NOERROR != result)
		throw result;

	// get line-in source
	MIXERLINE ml = {0};
	ml.cbStruct = sizeof(MIXERLINE);
	ml.dwComponentType = MIXERLINE_COMPONENTTYPE_SRC_LINE;
	result = mixerGetLineInfo((HMIXEROBJ) m_hmixer, &ml, MIXER_GETLINEINFOF_COMPONENTTYPE);
	if (MMSYSERR_NOERROR != result)
	{
		// fallback to microphone source, if line source not available
		ml.dwComponentType = MIXERLINE_COMPONENTTYPE_SRC_MICROPHONE;
		result = mixerGetLineInfo((HMIXEROBJ) m_hmixer, &ml, MIXER_GETLINEINFOF_COMPONENTTYPE);
		if (MMSYSERR_NOERROR != result)
		{
			MessageBox(NULL, "Couldn't get Line-In mixerComponent!", NULL, MB_OK);
			throw result;
		}
	}

	// setup mixer control details struct
	memset(&m_mcd, 0, sizeof(MIXERCONTROLDETAILS));
	m_mcd.cbStruct = sizeof(MIXERCONTROLDETAILS);
	m_mcd.cChannels = 1;

	// get line-in mute control
	MIXERLINECONTROLS mlc = {0};
	MIXERCONTROL mc = {0};
	mlc.cbStruct = sizeof(MIXERLINECONTROLS);
	mlc.dwLineID = ml.dwLineID;
	mlc.dwControlType = MIXERCONTROL_CONTROLTYPE_MUTE;
	mlc.cControls = 1;
	mlc.pamxctrl = &mc;
	mlc.cbmxctrl = sizeof(MIXERCONTROL);
	result = mixerGetLineControls((HMIXEROBJ) m_hmixer, &mlc, MIXER_GETLINECONTROLSF_ONEBYTYPE);
	if (MMSYSERR_NOERROR == result)
	{
		m_mcd.dwControlID = mc.dwControlID;
		m_mcd.cbDetails = sizeof(MIXERCONTROLDETAILS_BOOLEAN);
		m_mcd_muted.dwValue = 1;
		m_mcd_unmuted.dwValue = 0;

	} else {
		// fallback to line-in volume control
		mlc.dwControlType = MIXERCONTROL_CONTROLTYPE_VOLUME;
		result = mixerGetLineControls((HMIXEROBJ) m_hmixer, &mlc, MIXER_GETLINECONTROLSF_ONEBYTYPE);
		if (MMSYSERR_NOERROR != result)
		{
			MessageBox(NULL, "Couldn't get Volume mixerControl!", NULL, MB_OK);
			throw result;
		}

		m_mcd.dwControlID = mc.dwControlID;
		m_mcd.cbDetails = sizeof(MIXERCONTROLDETAILS_UNSIGNED);
		m_mcd_muted.dwValue = 0;
		m_mcd_unmuted.dwValue = 65535;
	}
}

CMixerCtrl::~CMixerCtrl()
{
	mixerClose(m_hmixer);
}

void CMixerCtrl::muteLineIn(BOOL muted)
{
	m_mcd.paDetails = (muted)?&m_mcd_muted:&m_mcd_unmuted;
	MMRESULT result = mixerSetControlDetails((HMIXEROBJ) m_hmixer, &m_mcd, MIXER_SETCONTROLDETAILSF_VALUE);
	if (MMSYSERR_NOERROR != result)
	{
		MessageBox(NULL, "Couldn't set mixerControl!", NULL, MB_OK);
	}
}


void CMixerCtrl::unMuteLineIn()
{
	muteLineIn(FALSE);
}
