/* $Id: Motorola.cpp,v 1.4 2007/02/01 11:25:06 olle Exp $
 *
 * Copyright (c) 2006, olle@toolcrypt.org
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 *    * Redistributions of source code must retain the above copyright notice,
 *      this list of conditions and the following disclaimer.
 *    * Redistributions in binary form must reproduce the above copyright notice,
 *      this list of conditions and the following disclaimer in the documentation
 *      and/or other materials provided with the distribution.
 *    * Neither the name of the <ORGANIZATION> nor the names of its contributors may
 *      be used to endorse or promote products derived from this software without 
 *      specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include "stdafx.h"

#include "Motorola.h"

#include "cmdWndProt.h"
#include "callinfo.h"

void
CMotorola::handleBit(bool bit)
{
    // Shift in the new bit into the 96 bit buffer.
    bitBuffer[0] <<= 1;
    bitBuffer[0] += (bitBuffer[1] & 0x80000000)?1:0;
    bitBuffer[1] <<= 1;
    bitBuffer[1] += (bitBuffer[2] & 0x80000000)?1:0;
    bitBuffer[2] <<= 1;
    if (!bit)
		bitBuffer[2] += 1;

	// update synctimer
	if (synctimer && ++synctimer > 160)
	{
		CMDWND_LOST_SYNC(cmdWnd);
		synctimer = 0;
	}

	// check for frame sync '10101100'
	// if found, we have a new frame
	if ((bitBuffer[2] & 0xFF000) == 0xAC000)
	{
		if (0) {
		CString msg;
		msg.Format("DEBUG| got frame %0.8X%0.8X%0.8X", bitBuffer[0], bitBuffer[1], bitBuffer[2]);
		CMDWND_ADD_LOGSTR(cmdWnd, (LPCTSTR)msg);
		}

		// deinterleave synched frame into buf[]
		unsigned char buf[10];
		memset(buf, 0, 10);
		unsigned char nBits = 0;
		for (int bit = 0; bit<19; ++bit)
			for (int pos = 0; pos<76; pos+=19)
			{
				if ((bitBuffer[(pos+bit)/32] >> ((31 - (pos+bit)%32))&0x1) == 0x1)
					buf[nBits/8] |= (1 << (7 - nBits%8));
				++nBits;
			}

		if (0) {
		CString msg;
		msg.Format("DEBUG| deinterleaved %u bits %0.2X%0.2X%0.2X%0.2X%0.2X%0.2X%0.2X%0.2X%0.2X%0.2X", nBits, buf[0], buf[1], buf[2], buf[3], buf[4], buf[5], buf[6], buf[7], buf[8], buf[9]);
		CMDWND_ADD_LOGSTR(cmdWnd, (LPCTSTR)msg);
		}

		// decode Motorola proprietary convolutional code
		unsigned char frame[5];
		memset(frame, 0, 5);
		unsigned short nErrors = 0;
		// extract bits into frame[], buf[] becomes syndrome
		for (unsigned char nBit = 0; nBit<76; nBit+=2)
		{
			if ((buf[nBit/8] >> ((7 - nBit%8))&0x1) == 0x1)
			{
				// extract data bit
				frame[(nBit/2)/8] |= (1 << (7 - (nBit/2)%8));

				// calculate syndrome
				buf[nBit/8] ^= (1 << (7 - nBit%8));
				buf[(nBit+1)/8] ^= (1 << (7 - (nBit+1)%8));
				buf[(nBit+3)/8] ^= (1 << (7 - (nBit+3)%8));
			}
		}
		// correct bit errors
		for (unsigned char nBit = 0; nBit<76; nBit+=2)
		{
			if (((buf[(nBit+1)/8] >> ((7 - (nBit+1)%8))&0x1) == 0x1) && ((buf[(nBit+3)/8] >> ((7 - (nBit+3)%8))&0x1) == 0x1))
			{
				frame[(nBit/2)/8] ^= (1 << (7 - (nBit/2)%8));
				buf[(nBit+1)/8] ^= (1 << (7 - (nBit+1)%8));
				buf[(nBit+3)/8] ^= (1 << (7 - (nBit+3)%8));
				++nErrors;
			}
		}

		if (0) {
			CString msg;
			msg.Format("DEBUG| corrected %u errors %0.2X%0.2X%0.2X%0.2X%0.2X", nErrors, frame[0], frame[1], frame[2], frame[3], frame[4]);
			CMDWND_ADD_LOGSTR(cmdWnd, (LPCTSTR)msg);
		}

		// calculate CRC for frame
		unsigned short shiftReg = 0x036E;
		unsigned short calculatedCRC = 0x0393;
		for (unsigned char nBit = 0; nBit < 27; ++nBit)
		{
			// update shift register
			if (shiftReg & 0x01) 
				shiftReg = (shiftReg >> 1) ^ 0x0225; 
			else 
				shiftReg = shiftReg >> 1;

			// add data bit
			if (((frame[nBit/8]>>(7 - nBit%8))&0x1) == 0x1)
				calculatedCRC ^= shiftReg;
		}
		
		// extract received CRC value
		unsigned short receivedCRC = 0;		
		for (unsigned char nBit = 0; nBit < 10; ++nBit)
		{
			receivedCRC <<= 1;
			if (((frame[(nBit+27)/8]>>(7 - (nBit+27)%8))&0x1) != 0x1)
				++receivedCRC;
		}

		// if frame with good CRC received, decode and process it
		if (calculatedCRC == receivedCRC)
		{
			codeWord_t cw;
			cw.id  = ((frame[0] << 8) + frame[1]) ^ 0xcc38;
			cw.grp = ((frame[2]>>7)&0x1)==0x0;
			cw.cmd = (((frame[2]<<3)&0x3f8) + ((frame[3]>>5)&0x7)) ^ 0x0d5;			
			processCodeWord(cw);
			++goodframes;

			// report found sync
			CMDWND_GOT_SYNC(cmdWnd);
			synctimer = 1;
		}
		else
		{
			++badframes;
			if (0) {
				CString msg;
				msg.Format("ERROR| Bad frame, %.2u bit errors       | %0.2X%0.2X%0.2X%0.2X%0.2X %0.4X %0.4X", nErrors, frame[0], frame[1], frame[2], frame[3], frame[4], calculatedCRC, receivedCRC);
				CMDWND_ADD_LOGSTR(cmdWnd, (LPCTSTR)msg);
			}
		}
	}
}

void CMotorola::setSystemID(unsigned short id)
{
	if (id != currentsysid)
	{
		snabel_addr_info_t *addr = new snabel_addr_info_t;
		addr->system = currentsysid = id;
		addr->fleet = addr->addr = 0;
		CMDWND_REPORT_ADDR(cmdWnd, addr);
	}
}

void CMotorola::processCodeWord(codeWord_t cwIn)
{
	if (0) {
		CString msg;
		msg.Format("DEBUG| id=%0.4X grp=%u cmd=%0.3X", cwIn.id, cwIn.grp, cwIn.cmd);
		CMDWND_ADD_LOGSTR(cmdWnd, (LPCTSTR)msg);
	}

	// skip idle codeword
	if (cwIn.cmd == 0x02f8)
		return;

	// skip site IDs
	if (cwIn.id == 0xffff && cwIn.cmd >= 0x0360 && cwIn.cmd < 0x03a0)
	{
		CString msg;
		msg.Format("SITE | System 0x%0.4X Site %u", currentsysid, cwIn.cmd - 0x0360);
		CMDWND_ADD_LOGSTR(cmdWnd, (LPCTSTR)msg);
		return;
	}

	// skip scan marker
	if (cwIn.cmd == 0x032b && !cwIn.grp)
	{
		// id contains the system ID
		setSystemID(cwIn.id);
		return;
	}

	// keep a sliding window of 3 codewords
	codeWord[0] = codeWord[1];
	codeWord[1] = codeWord[2];
	codeWord[2] = cwIn;

	// codeword 308 and group bits, we have a command sequence
	if (codeWord[0].cmd == 0x0308 && codeWord[0].grp && codeWord[1].grp)
	{
		// system announce (308/30b)
		if ( codeWord[1].cmd == 0x030b)
		{
			// report system id
			setSystemID(codeWord[0].id);

			// set current control channel
			if ((codeWord[1].id & 0xfc00) == 0x2800)
				currentcc = codeWord[1].id - 0x2800;

			CString msg;
			msg.Format("SYSAN| System 0x%0.4X on control channel %u", currentsysid, currentcc);
			CMDWND_ADD_LOGSTR(cmdWnd, (LPCTSTR)msg);
		}

		// radio affiliation (308/310)
		if (codeWord[1].cmd == 0x0310)
		{
			snabel_addr_info_t *radio = new snabel_addr_info_t;
			snabel_addr_info_t *group = new snabel_addr_info_t;
			radio->system = group->system = currentsysid;
			radio->fleet = group->fleet = 1; // XXX: dummy fleet
			radio->addr = codeWord[0].id + 700000;
			group->addr = codeWord[1].id;
			CMDWND_REPORT_ADDR(cmdWnd, radio);
			CMDWND_REPORT_ADDR(cmdWnd, group);

			CString msg;
			msg.Format("AFFIL| Radio %u affiliated to group %u", codeWord[0].id, codeWord[1].id);
			CMDWND_ADD_LOGSTR(cmdWnd, (LPCTSTR)msg);
		}
		return;
	}

	// this is group call setup for UHF/VHF systems, cmd contains channel number
	// (000-17b == low) (17c-2f7 == high) centered around base frequency == 17c
	// right now this assumes that outputs are on high frequencies and inputs on low
	if (codeWord[1].cmd < 0x02f8 && codeWord[1].cmd > 0x017b)
	{
		// setup new call
		snabel_call_info_t *call = new snabel_call_info_t;
		call->calltype = SNABEL_CALL_TYPE_ANALOG_VOICE;
		call->ctrlchnl = currentcc;
		call->calladdr.system = call->fromaddr.system = currentsysid;
		call->calladdr.fleet = call->fromaddr.fleet = 1; // XXX: dummy fleet

		// is the last codeword the input frequency?
		if (codeWord[0].cmd < 0x017c)
		{
			// then it has the originating ID
			call->calladdr.addr = codeWord[1].id;
			call->fromaddr.addr = codeWord[0].id + 700000;
			call->callchnl = codeWord[1].cmd;

		} else {
			// otherwise there is no originating ID, this is a single codeword call indication
			call->calladdr.addr = call->fromaddr.addr = codeWord[1].id;
			call->callchnl = codeWord[1].cmd;
		}

		CString msg;
		msg.Format("VCALL| Call to %u from %u on channel %u", call->calladdr.addr, call->fromaddr.addr, call->callchnl);
		CMDWND_ADD_LOGSTR(cmdWnd, (LPCTSTR)msg);

		// report call
		CMDWND_REPORT_CALL(cmdWnd, call);
		return;
	}
}