/* $Id: decoderThread.cpp,v 1.8 2007/02/01 10:28:00 olle Exp $
 *
 * Copyright (c) 2006, olle@toolcrypt.org
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 *    * Redistributions of source code must retain the above copyright notice,
 *      this list of conditions and the following disclaimer.
 *    * Redistributions in binary form must reproduce the above copyright notice,
 *      this list of conditions and the following disclaimer in the documentation
 *      and/or other materials provided with the distribution.
 *    * Neither the name of the <ORGANIZATION> nor the names of its contributors may
 *      be used to endorse or promote products derived from this software without 
 *      specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include "stdafx.h"

#include "DecoderThread.h"

CDecoderThread::CDecoderThread (CWnd* cmdWnd, DWORD sampleRate)
		: nmt450(cmdWnd, sampleRate)
		, mpt1327(cmdWnd, sampleRate)
		, moto3600(cmdWnd, sampleRate, 3600)
		, hWaveIn(0)
		, suspend(FALSE)
{
	WAVEFORMATEX waveFormatEx = {0};
	MMRESULT res = 0;

	// open the wave input device, bind it to the static callback
    waveFormatEx.wFormatTag = WAVE_FORMAT_PCM;
    waveFormatEx.nChannels = 1; //TODO: let user choose channel?
    waveFormatEx.wBitsPerSample = 16;
    waveFormatEx.nSamplesPerSec = sampleRate;
    waveFormatEx.nBlockAlign = waveFormatEx.nChannels * waveFormatEx.wBitsPerSample/8;
    waveFormatEx.nAvgBytesPerSec = waveFormatEx.nSamplesPerSec * waveFormatEx.nChannels * waveFormatEx.wBitsPerSample/8;
    waveFormatEx.cbSize = 0;
	res = waveInOpen(&hWaveIn, WAVE_MAPPER, &waveFormatEx, (DWORD_PTR)waveInCallback, (DWORD_PTR)this, CALLBACK_FUNCTION);
	// TODO: check for errors

	// create a sample buffer
	memset(&waveHeader, 0, sizeof(waveHeader));
	waveHeader.dwBufferLength = waveFormatEx.nSamplesPerSec / 10;
	waveHeader.lpData = (char*)LocalAlloc(0, waveHeader.dwBufferLength);
	// TODO: check for errors
	res = waveInPrepareHeader(hWaveIn, &waveHeader, sizeof waveHeader);
	// TODO: check for errors
	res = waveInAddBuffer(hWaveIn, &waveHeader, sizeof waveHeader);
	// TODO: check for errors

	// start recording
	res = waveInStart(hWaveIn);
	// TODO: check for errors
}

CDecoderThread::~CDecoderThread ()
{
	MMRESULT res = 0;

	// suspend decoding
	suspend = TRUE;

	// stop recording
	res = waveInStop(hWaveIn);
	// TODO: check for errors
	res = waveInClose(hWaveIn);
	// TODO: check for errors

	// free samplebuffer
	LocalFree(waveHeader.lpData);
}


void CALLBACK CDecoderThread::waveInCallback(HWAVEIN hwi, UINT uMsg, DWORD_PTR dwInstance, DWORD_PTR dwParam1, DWORD_PTR dwParam2)
{
	WAVEHDR *lpHdr = (WAVEHDR *)dwParam1;
	CDecoderThread *thread = (CDecoderThread*)dwInstance;
	switch (uMsg)
	{		
	case WIM_OPEN:	
		break;
	case WIM_CLOSE:
		break;
	case WIM_DATA:
		thread->sampleDelivery((PUSHORT)lpHdr->lpData, lpHdr->dwBytesRecorded / 2 /* number of 16-bit samples */);
		waveInUnprepareHeader(hwi, lpHdr, sizeof *lpHdr);
		waveInPrepareHeader(hwi, lpHdr, sizeof *lpHdr);
		waveInAddBuffer(hwi, lpHdr, sizeof *lpHdr);
		break;
	default:
		break;
	}
}


void CDecoderThread::sampleDelivery(PUSHORT data, DWORD nSamples)
{
	// deliver samples to the system decoders
	if (!suspend)
	{
		nmt450.inputData(data, nSamples);
		mpt1327.inputData(data, nSamples);
		moto3600.inputData(data, nSamples);
	}
}