/* $Id: snabelDlg.cpp,v 1.23 2007/02/04 11:35:27 olle Exp $
 *
 * Copyright (c) 2006, olle@toolcrypt.org
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 *    * Redistributions of source code must retain the above copyright notice,
 *      this list of conditions and the following disclaimer.
 *    * Redistributions in binary form must reproduce the above copyright notice,
 *      this list of conditions and the following disclaimer in the documentation
 *      and/or other materials provided with the distribution.
 *    * Neither the name of the <ORGANIZATION> nor the names of its contributors may
 *      be used to endorse or promote products derived from this software without 
 *      specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include "stdafx.h"

#include "SnabelApp.h"
#include "SnabelDlg.h"
#include "PrefDlg.h"

#include "UnidenScanner.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif


// CAboutDlg dialog used for App About

class CAboutDlg : public CDialog
{
public:
	CAboutDlg();

// Dialog Data
	enum { IDD = IDD_ABOUTBOX };

	protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support

// Implementation
protected:
	DECLARE_MESSAGE_MAP()
};

CAboutDlg::CAboutDlg() : CDialog(CAboutDlg::IDD)
{
}

void CAboutDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
}

BEGIN_MESSAGE_MAP(CAboutDlg, CDialog)
END_MESSAGE_MAP()


// CSnabelDlg dialog

CSnabelDlg::CSnabelDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CSnabelDlg::IDD, pParent)
	, m_Mute(FALSE)
    , m_FollowCalls(FALSE)
    , m_ScanOnClear(FALSE)
	, m_AutoAddIds(FALSE)
	, m_ControlBacklight(FALSE)
	, m_ComPort(0)
	, m_MemoryLocation(0)
	, m_decoderThread(0)
	, m_SampleRate(0)
	, m_ReturnToCC(FALSE)
{
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

CSnabelDlg::~CSnabelDlg()
{
	if (m_decoderThread)
		delete m_decoderThread;

	if (m_cmdWnd)
	{
		// give CCmdWnd a chance to cleanup
		m_cmdWnd->DestroyWindow();
		delete m_cmdWnd;
	}
}

void CSnabelDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_LOGLIST, c_LogList);
	DDX_Control(pDX, IDC_STATIC_FRAME, c_Frame);
	DDX_Control(pDX, IDC_CLEAR_BUTTON, c_ClearButton);
}

BEGIN_MESSAGE_MAP(CSnabelDlg, CDialog)
	ON_WM_CREATE()
	ON_WM_SYSCOMMAND()
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_BN_CLICKED(IDC_PREFS_BUTTON, OnPrefsButton)
	ON_BN_CLICKED(IDC_CLEAR_BUTTON, OnClearButton)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


// CSnabelDlg message handlers
void CSnabelDlg::OnOK()
{
	// do nothing
}

void CSnabelDlg::OnCancel()
{
	// save addressbook data
	char Path[MAX_PATH];
  	GetCurrentDirectory(MAX_PATH-15, Path);
	strcat(Path, "\\addrbook.ini");
	c_AddrBook.SaveData(Path);

	// let user abort exit
	if(AfxMessageBox("Are you sure you want to quit?   ",MB_YESNO)==IDYES)
		EndDialog(0);	
}

void CSnabelDlg::InitPreferences()
{
	char Path[MAX_PATH];
  	GetCurrentDirectory(MAX_PATH-15, Path);
	strcat(Path, "\\snabel.ini");

	UINT samplerate = GetPrivateProfileInt("audio", "samplerate", 48000, Path);
	if (m_SampleRate != samplerate)
	{
		m_SampleRate = samplerate;

		// start/restart decoder thread
		if (m_decoderThread)
		{
			delete m_decoderThread;
			c_LogList.AddString("Restarting decoder thread.");
		}
		else
			c_LogList.AddString("Starting decoder thread.");

		try {
			m_decoderThread = new CDecoderThread(m_cmdWnd, m_SampleRate);
		} catch(...) {
			c_LogList.AddString("ERROR: Couldn't start collecting data, usupported samplerate?");
		}		
	}
	
    m_Mute = (GetPrivateProfileInt("audio", "mute", 1, Path) == 1);


	UINT receivertype = GetPrivateProfileInt("tracking", "receiver_type", 0, Path);
	UINT comport = GetPrivateProfileInt("tracking", "comport", 0, Path);
	UINT memory = GetPrivateProfileInt("tracking", "memory", 0, Path);
	if (receivertype != m_ReceiverType || comport != m_ComPort || memory != m_MemoryLocation)
	{
		m_ReceiverType = receivertype;
		m_ComPort = comport;
		m_MemoryLocation = memory;
		
		// init receiver
		m_cmdWnd->initReceiver(m_ReceiverType, m_ComPort, m_MemoryLocation);
	}
	
    m_FollowCalls = (GetPrivateProfileInt("tracking", "follow_calls", 1, Path) == 1);
    m_ScanOnClear = (GetPrivateProfileInt("tracking", "scan_on_clear", 0, Path) == 1);
	m_AutoAddIds = (GetPrivateProfileInt("tracking", "auto_add", 1, Path) == 1);
	m_ControlBacklight = (GetPrivateProfileInt("tracking", "backlight", 0, Path) == 1);
	m_ReturnToCC = (GetPrivateProfileInt("tracking", "return_to_cc", 0, Path) == 1);
}

BOOL CSnabelDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Add "About..." menu item to system menu.

	// IDM_ABOUTBOX must be in the system command range.
	ASSERT((IDM_ABOUTBOX & 0xFFF0) == IDM_ABOUTBOX);
	ASSERT(IDM_ABOUTBOX < 0xF000);

	CMenu* pSysMenu = GetSystemMenu(FALSE);
	if (pSysMenu != NULL)
	{
		CString strAboutMenu;
		strAboutMenu.LoadString(IDS_ABOUTBOX);
		if (!strAboutMenu.IsEmpty())
		{
			pSysMenu->AppendMenu(MF_SEPARATOR);
			pSysMenu->AppendMenu(MF_STRING, IDM_ABOUTBOX, strAboutMenu);
		}
	}

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon


	// move AddrBook into static frame area
	CRect rc;
	c_Frame.GetWindowRect(rc);
	ScreenToClient(rc);
	c_Frame.ShowWindow(SW_HIDE);
	if(c_AddrBook.GetSafeHwnd())
			c_AddrBook.MoveWindow(rc);

	// initialize controls
	CFont *monospaced = new CFont();
	monospaced->CreatePointFont(90, "Lucida Console");
	c_LogList.SetFont(monospaced);	

	// load addressbook data
	char Path[MAX_PATH];
  	GetCurrentDirectory(MAX_PATH-15, Path);
	strcat(Path, "\\addrbook.ini");
	c_AddrBook.LoadData(Path);

	// create command window
	m_cmdWnd = new CCmdWnd(this);
	if (!m_cmdWnd->Create())
	{
		MessageBox("Error creating command window!");
		return FALSE;
	}

	// load preferences and init system
	InitPreferences();
	c_LogList.AddString("Initialized and running.");

	return TRUE;  // return TRUE  unless you set the focus to a control
}

int CSnabelDlg::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	if (CDialog::OnCreate(lpCreateStruct) == -1)
		return -1;

	c_AddrBook.Create(WS_CHILD | WS_VISIBLE | WS_TABSTOP, CRect(0,0,0,0),this,0x1001);	
	c_AddrBook.Initialize();
	/*
	ExtendedStyles supported:
	LVS_EX_TRACKSELECT
	LVS_EX_GRIDLINES
	LVS_EX_FLATSB
	LVS_EX_CHECKBOXES
	*/
	c_AddrBook.SetExtendedStyle(LVS_EX_CHECKBOXES);
	c_AddrBook.ModifyStyleEx(0,WS_EX_CLIENTEDGE);

	return 0;
}


void CSnabelDlg::OnSysCommand(UINT nID, LPARAM lParam)
{
	if ((nID & 0xFFF0) == IDM_ABOUTBOX)
	{
		CAboutDlg dlgAbout;
		dlgAbout.DoModal();
	}
	else
	{
		CDialog::OnSysCommand(nID, lParam);
	}
}


// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void CSnabelDlg::OnPaint() 
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, reinterpret_cast<WPARAM>(dc.GetSafeHdc()), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

// The system calls this function to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR CSnabelDlg::OnQueryDragIcon()
{
	return static_cast<HCURSOR>(m_hIcon);
}


// bring up the preferences dialog
void CSnabelDlg::OnPrefsButton()
{
	// do the preferences dialog
	CPrefDlg prefs;
	prefs.DoModal();

	// re-load preferences from file
	InitPreferences();
}

// clear down monitored call
void CSnabelDlg::OnClearButton()
{
	CMDWND_CLEAR_CALL_MANUAL(m_cmdWnd);
}

// CCmdWnd 

BEGIN_MESSAGE_MAP(CCmdWnd, CWnd)
	ON_WM_DESTROY()
	ON_MESSAGE(WM_USER, OnUser)
	ON_WM_TIMER()
END_MESSAGE_MAP()

CCmdWnd::CCmdWnd(CSnabelDlg *pParent)
{
	m_callTimer = 0;
	m_receiverCtrl = NULL;
	m_skipCallAddr.system = m_skipCallAddr.fleet = m_skipCallAddr.addr = 0;

	if (!pParent)
		throw;
	m_pParent = pParent;
}

CCmdWnd::~CCmdWnd()
{
	if (m_receiverCtrl)
		delete m_receiverCtrl;
}

BOOL CCmdWnd::Create()
{
	RECT rect;
	const RECT &rectref = rect; 
	return CWnd::Create(NULL, "snabel command window", WS_DISABLED, rectref, m_pParent, 0);
}

BOOL CCmdWnd::initReceiver(UINT type, UINT port, UINT memory)
{
	if (m_receiverCtrl)
	{
		delete m_receiverCtrl;
		m_receiverCtrl = NULL;
	}

	if (port)
	{
		CString msg;
		try {
			// create a receiver of the correct type
			switch (type)
			{
			case 2:
				m_receiverCtrl = new CUnidenDynamic(port);
				msg.Format("Initialized Uniden Experimental on serial port %u.", port);
				break;
			case 1:
				m_receiverCtrl = new CUniden250(port, memory);
				msg.Format("Initialized Uniden 250/785 on serial port %u.", port);
				break;
			case 0:
			default:
				m_receiverCtrl = new CUniden780(port, memory);
				msg.Format("Initialized Uniden 245/780 on serial port %u.", port);
			}

			// init the receiver
			if (!m_receiverCtrl->initReceiver())
			{
				msg.Format("Initialization of receiver failed!");
				delete m_receiverCtrl;
				m_receiverCtrl = NULL;
			}

			// log result
			CMDWND_ADD_LOGSTR(this, msg);

		} catch (const char *errmsg) {
			msg.Format("ERROR initializing receiver on serial port %u: %s", port, errmsg);
			CMDWND_ADD_LOGSTR(this, msg);
			return FALSE;
		} catch (...) {
			msg.Format("ERROR initializing receiver on serial port %u.", port);
			CMDWND_ADD_LOGSTR(this, msg);
			return FALSE;
		}
	}
	return TRUE;
}


void CCmdWnd::OnDestroy()
{
	// unmute line-in on exit
	if (m_pParent->m_Mute)
		m_mixer.unMuteLineIn();
}

LRESULT CCmdWnd::OnUser(WPARAM wParam, LPARAM lParam)
{
	switch(wParam)
	{

	case CMDWND_WPARAM_ADD_LOGSTR:
		if (lParam)
		{
			// keep log trimmed at 1000 lines
			if (m_pParent->c_LogList.GetCount() > 1000)
				m_pParent->c_LogList.DeleteString(0);

			// add new string to log
			int newindex = m_pParent->c_LogList.AddString((LPCTSTR)lParam);
			free((LPVOID)lParam);
			if (newindex < 0)
			{
				MessageBox("Error adding string to LogList!");
			} else {
				// position listbox at newly inserted entry
				m_pParent->c_LogList.SetCaretIndex(newindex);
			}

		} else {
			MessageBox("Invalid parameters to CMDWND_ADD_LOGSTR!");
		}
		break;


	case CMDWND_WPARAM_REPORT_CALL:
		if (lParam)
		{
			snabel_call_info_t *call = (snabel_call_info_t *)lParam;

			// Add addresses to addressbook, if allowed
			if (m_pParent->m_AutoAddIds)
			{
				m_pParent->c_AddrBook.AddUniqueAddr(call->fromaddr);
				m_pParent->c_AddrBook.AddUniqueAddr(call->calladdr);
			}

			// start monitoring only if no call currently in progress
			// and destination is not a manually cleared-down call
			if ( m_callTimer==0 && 
				!(m_skipCallAddr.addr == call->calladdr.addr && 
				  m_skipCallAddr.fleet == call->calladdr.fleet &&
				  m_skipCallAddr.system == call->calladdr.system)
				)
				// TODO: call priorities?

			// tune reciever if present and allowed
			if (m_receiverCtrl && m_pParent->m_FollowCalls)
			{
				// lookup destination address in AddrBook
				int checked = m_pParent->c_AddrBook.IsAddrChecked(call->calladdr, TRUE);

				// if destination address not found, lookup sender
				if (checked==-1) 
					checked = m_pParent->c_AddrBook.IsAddrChecked(call->fromaddr, TRUE);

				// only monitor calls with checked addresses
				if (checked==1)
				{
					snabel_system_params_t params;
					snabel_system_params_t &paramref = params;
					if (!m_sysParams.Lookup(call->calladdr.system, paramref))
					{
						m_pParent->c_LogList.AddString("Couldn't tune receiver, missing system parameters!");
					} else {

						// set backlight
						if (m_pParent->m_ControlBacklight)
							m_receiverCtrl->setBacklight(m_receiverCtrl->High);

						// tune receiver
						m_receiverCtrl->setFrequency((params.base + (DOUBLE)((call->callchnl - params.offset) * (DOUBLE)params.step)), (params.step < 0.020));

						// if controlchannel not specified, use last seen for this system
						if (!call->ctrlchnl)
							call->ctrlchnl = params.channel;

						// save current call info
						m_currentCall = *call;

						// set call info text
						CString text = "";
						if (call->fromaddr.addr && !(call->fromaddr.addr == call->calladdr.addr && call->fromaddr.fleet == call->calladdr.fleet && call->fromaddr.system == call->calladdr.system))
						{
							text = m_pParent->c_AddrBook.GetAddrName(call->fromaddr);
							if (call->calladdr.addr)
								text.Append(" > ");
						}
						if (call->calladdr.addr)
							text.Append(m_pParent->c_AddrBook.GetAddrName(call->calladdr));
						m_receiverCtrl->setText(text);

						// set fleet info text
						call->calladdr.addr = 0;
						text = m_pParent->c_AddrBook.GetAddrName(call->calladdr);
						m_receiverCtrl->setText2(text);

						// start call timer
						m_callTimer = 1;
						SetTimer(1, 1000, NULL);

						// enable clear button
						m_pParent->c_ClearButton.EnableWindow(TRUE);
					}
				}
			}

			delete call;
		} else {
			MessageBox("Invalid parameters to CMDWND_REPORT_CALL!");
		}
		break;

	case CMDWND_WPARAM_CLEAR_CALL_MANUAL:
		{
			// duplicate current call info
			snabel_call_info_t *call = new snabel_call_info_t;
			call->calltype = m_currentCall.calltype;
			call->callchnl = m_currentCall.callchnl;
			call->ctrlchnl = m_currentCall.ctrlchnl;
			call->fromaddr = m_currentCall.fromaddr;
			call->calladdr = m_currentCall.calladdr;

			// save info about skipped call
			m_skipCallAddr = m_currentCall.calladdr;

			// don't return to skipped call for another 10 seconds
			KillTimer(2);
			SetTimer(2, 10000, NULL);

			// intentional fall-through to CMDWND_WPARAM_CLEAR_CALL
			lParam = (LPARAM)call;
		}

	case CMDWND_WPARAM_CLEAR_CALL:
		if (lParam)
		{
			snabel_call_info_t *call = (snabel_call_info_t *)lParam;

			// if call in progress, tune reciever if present and allowed
			if (m_callTimer && m_receiverCtrl && m_pParent->m_FollowCalls)
			{
				// set backlight
				if (m_pParent->m_ControlBacklight)
					m_receiverCtrl->setBacklight(m_receiverCtrl->Low);

				// start scanning ...
				if (m_pParent->m_ScanOnClear)
				{
					m_receiverCtrl->startScan();

				// ... or return to control channel
				} else if (m_pParent->m_ReturnToCC) {

					// return to control channel
					snabel_system_params_t params;
					snabel_system_params_t &paramref = params;
					if (!m_sysParams.Lookup(call->calladdr.system, paramref))
					{
						m_pParent->c_LogList.AddString("Couldn't tune receiver, missing system parameters!");
					} else {
						m_receiverCtrl->setFrequency((params.base + (DOUBLE)((call->ctrlchnl - params.offset) * (DOUBLE)params.step)), (params.step < 0.020));
					}

				// otherwise we park channel on freq 0.0
				} else {
					m_receiverCtrl->setFrequency(0);
				}

				// set System info text
				CString text;
				call->calladdr.fleet = call->calladdr.addr = 0;
				text.Append(m_pParent->c_AddrBook.GetAddrName(call->calladdr));
				m_receiverCtrl->setText(text);

				// clear Fleet info text
				m_receiverCtrl->setText2(NULL);

				// log call cleardown
				CString msg;
				msg.Format("     | Call duration: %4um %2us        |", m_callTimer/60, m_callTimer%60);
				m_pParent->c_LogList.AddString(msg);

				// clear call timer
				KillTimer(1);
				m_callTimer = 0;

				// set backlight
				if (m_pParent->m_ControlBacklight)
					m_receiverCtrl->setBacklight(m_receiverCtrl->Off);

				// disable clear button
				m_pParent->c_ClearButton.EnableWindow(FALSE);
			}

			delete call;
		} else {
			MessageBox("Invalid parameters to CMDWND_CLEAR_CALL!");
		}
		break;


	case CMDWND_WPARAM_GOT_SYNC:

		// mute line-in
		if (m_pParent->m_Mute)
			m_mixer.muteLineIn();

		break;


	case CMDWND_WPARAM_LOST_SYNC:

		// unmute line-in
		if (m_pParent->m_Mute)
			m_mixer.unMuteLineIn();

		break;

	case CMDWND_WPARAM_REPORT_ADDR:

		if (lParam)
		{
			snabel_addr_info_t *addr = (snabel_addr_info_t *)lParam;

			// if this is a system report
			if (addr->system && !addr->addr && !addr->fleet)
			{
				// stop call timer, don't clear it (setting m_callTimer=0 would prevent action on a CLEAR message)
				KillTimer(1);

				// if params for this system not loaded yet
				snabel_system_params_t params;
				snabel_system_params_t &paramref = params;
				if (!m_sysParams.Lookup(addr->system, paramref))
				{
					// get system parameters from file
					char Path[MAX_PATH];
	  				GetCurrentDirectory(MAX_PATH, Path);
					strcat(Path, "\\snabel.ini");
					CString sysBase;
					sysBase.Format("sysBase_%u", addr->system);
					CString sysStep;
					sysStep.Format("sysStep_%u", addr->system);
					CString sysOffset;
					sysOffset.Format("sysOffset_%u", addr->system);
					params.base = ((DOUBLE)GetPrivateProfileInt("systems", sysBase, 4200000, Path) / 10000);
					params.step = ((DOUBLE)GetPrivateProfileInt("systems", sysStep, 125, Path) / 10000);
					params.offset = (USHORT)GetPrivateProfileInt("systems", sysOffset, 0, Path);
					params.channel = 0;

					// save them for later
					m_sysParams.SetAt(addr->system, params);
				}

				// calculate current control channel number and save it in system parameters
				if (m_receiverCtrl)
				{
					DOUBLE freq = m_receiverCtrl->getFrequency();
					if (freq > params.base) {
						params.channel = (UINT)((freq - params.base)/params.step) + params.offset;
						m_sysParams.SetAt(addr->system, params);
					}
				}

				// set System info text
				if (m_receiverCtrl && m_pParent->m_FollowCalls)
				{
					CString text;
					text.Append(m_pParent->c_AddrBook.GetAddrName(*addr));
					m_receiverCtrl->setText(text);
				}
			}

			// add the address to addressbook, if allowed
			if (m_pParent->m_AutoAddIds)
				m_pParent->c_AddrBook.AddUniqueAddr(*addr);

			delete addr;
		}
		break;

	default:
		break;
	}
	return 0;
}


void CCmdWnd::OnTimer(UINT id)
{
	switch(id)
	{
	// squelch-triggered call timeout
	case 1:
		// if call timer disabled, commit suicide
		if (!m_callTimer)
		{
			KillTimer(id);
			return;
		}

		// check for carrier every 5 seconds
		if ((++m_callTimer % 5)==0)
		{		
			// check receiver squelch
			if (!(m_receiverCtrl && m_receiverCtrl->isSquelchOpen()))
			{
				// if squelched, clear down the call
				snabel_call_info_t *call = new snabel_call_info_t;
				*call = m_currentCall;
				CMDWND_CLEAR_CALL(this, call);

			}
		}
		break;

	// manually cleared-call life-timer
	case 2:
		// delete manually cleared-down call info
		m_skipCallAddr.system = m_skipCallAddr.fleet = m_skipCallAddr.addr = 0;
		KillTimer(id);
		break;

	default:
		break;
	}
}