/* $Id: snabelDlg.h,v 1.12 2007/02/04 11:35:27 olle Exp $
 *
 * Copyright (c) 2006, olle@toolcrypt.org
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 *    * Redistributions of source code must retain the above copyright notice,
 *      this list of conditions and the following disclaimer.
 *    * Redistributions in binary form must reproduce the above copyright notice,
 *      this list of conditions and the following disclaimer in the documentation
 *      and/or other materials provided with the distribution.
 *    * Neither the name of the <ORGANIZATION> nor the names of its contributors may
 *      be used to endorse or promote products derived from this software without 
 *      specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */


#pragma once

#include "afxwin.h"
#include "afxcmn.h"

#include "MixerCtrl.h"
#include "AddrBookCtrl.h"
#include "ReceiverCtrl.h"
#include "DecoderThread.h"
#include "cmdWndProt.h"
#include "callinfo.h"

// forward definition of CCmdWnd
class CCmdWnd;


// CSnabelDlg dialog
class CSnabelDlg : public CDialog
{
// Construction
public:
	CSnabelDlg(CWnd* pParent = NULL);	// standard constructor
	~CSnabelDlg();	// standard destructor

// Dialog Data
	enum { IDD = IDD_SNABEL_DIALOG };

// Implementation
protected:
	HICON m_hIcon;

	// children
	CCmdWnd *m_cmdWnd;
	CDecoderThread *m_decoderThread;

	// members
	void InitPreferences();

	// DDX/DDV support
	virtual void DoDataExchange(CDataExchange* pDX);

	// message map functions
	virtual void OnOK();
	virtual void OnCancel();
	virtual BOOL OnInitDialog();
	afx_msg int OnCreate(LPCREATESTRUCT lpCreateStruct);
	afx_msg void OnSysCommand(UINT nID, LPARAM lParam);
	afx_msg void OnPaint();
	afx_msg HCURSOR OnQueryDragIcon();
	afx_msg void OnPrefsButton();
	afx_msg void OnClearButton();
	DECLARE_MESSAGE_MAP()

public:
	// preferences
	UINT m_SampleRate;
	BOOL m_Mute;
	BOOL m_FollowCalls;
	BOOL m_ScanOnClear;
	UINT m_ReceiverType;
	UINT m_ComPort;
	BOOL m_AutoAddIds;
	UINT m_MemoryLocation;
	BOOL m_ControlBacklight;	
	BOOL m_ReturnToCC;

	// controls
	CListBox c_LogList;
	CStatic	c_Frame;
	CAddrBookCtrl c_AddrBook;
	CButton c_ClearButton;
};

// CCmdWnd - Window with message queue separate from GUI thread

class CCmdWnd : public CWnd
{
// Construction
public:
	CCmdWnd(CSnabelDlg *pParent);	// standard constructor
	~CCmdWnd();	// standard destructor

	BOOL Create(void);
    BOOL initReceiver(UINT type, UINT port, UINT memory);	

protected:

	// resources
	CMixerCtrl m_mixer;
	CReceiverCtrl *m_receiverCtrl;

	// message map functions
	afx_msg void OnDestroy();
	afx_msg LRESULT OnUser(WPARAM wParam, LPARAM lParam);
	afx_msg void OnTimer(UINT id);
	DECLARE_MESSAGE_MAP()

private:

	// implementation details
	typedef struct snabel_system_params_s
	{
		double base;
		double step;
		signed short offset;
		UINT channel;
	} snabel_system_params_t;

	CMap<UINT,UINT, snabel_system_params_t, snabel_system_params_t&> m_sysParams;
	UINT m_callTimer;
	snabel_addr_info_t m_skipCallAddr;
	snabel_call_info_t m_currentCall;
	CSnabelDlg *m_pParent;
};

